# Licensed Materials - Property of IBM
# Rational ClearCase
# (C) Copyright IBM Corp. 1996, 2014.  All Rights Reserved.
# US Government Users Restricted Rights -
# Use, duplication or disclosure restricted by GSA ADP Schedule
# Contract with IBM Corp


require "vob_snap_machdep.pl";


#
# constants
#   RECOVER_PROGRAM
#     The name of this program 
# global data
#   Data obtained from the argument list:
#     $recover_arg_vob_tag 
#       The vob tag of the vob being recovered. 
#     $recover_arg_snap_storage
#       The snapped data path with '/replica_uuid' appended. This is
#       where vob_snapshot places it.
#     $recover_arg_dest_storage
#       The destination storage location. This may or may not be the same
#       as recover_rgy_storage below.
#     $recover_arg_temp_storage
#       Path for temporary storage of the vob storage data.
#   Data obtained for recover_arg_vob_tag from the registry.
#     $recover_rgy_storage
#       The local storage path. This is the current local storage path. The
#       user may specify a new storage location on the command line through
#       the recover_arg_dest_storage argument.
#     $recover_rgy_replica_uuid
#       The replica uuid
#     $recover_rgy_family_uuid
#       The family uuid
#     $recover_rgy_host
#       The replica host
#   Other data
#     $recover_var_directory 
#       Holds the hosts /opt/rational/clearcase/atria directory.
#     $recover_host_name
#       The name of this host
#     @recover_cur_function
#        A stack containing a call trace. The current function is pushed
#        onto @recover_cur_function when a function is entered by 
#        &recover_set_cur_function and popped off on leaving.
#     $recover_scenario
#       The scenario inferred by the arguments.
#     $recover_dbcheck_done
#       A clean dbcheck has been done by this script
#     $recover_state_file
#       Path to the state file.
#     $recover_state_is_open 
#       Set when the state file is open for business.
#     %recover_state
#       Contains the state left by the last run. See the comment with 
#       &recover_st_log.
#   These variables tell the cleanup function about things that must 
#   be cleaned up or undone before going away.
#     $recover_cl_unregister
#       Unregister the replica
#     $recover_cl_db_protect
#       Reprotect the database.
# Functions descriptions
#   Those functions contained in this module.
#     &recover_check_clearcase()
#       Check to see if clearcase is installed
#     &recover_parse_arguments()
#       Check arguments
#     &recover_set_scenario()
#       Examine the arguments and set recover_scenario
#     &recover_get_vob_info()
#       Set registry globals
#     &recover_check_snap()
#       Check the integrety if the snapshot area.
#     &vob_insert_snapped_data()
#       Install a snapped area into the vob storage.
#     &recover_invoke_checkvob()
#     &recover_checkvob_status()
#     &recover_restore_from_replica()
#
# Functions contained in mdep_machdep.pl
# 
#     &mdep_command()
#       Process the command passed to it and return the output in an
#       array. If the command fails the first element of the array
#       is specified as $MDEP_COMMAND_FAILURE.
#     &mdep_prompt()
#       Put out a prompt string to the user and get a reply.
# 

$RECOVER_E_CCASE_UP_CHECK =
"Check for clearcase running failed\n";
$RECOVER_E_CCASE_MUST_BE_UP =
"Clearcase is expected to be running and it is not.\n";
$RECOVER_E_CLEARCASE_NOT_INSTALLED =
"Clearcase does not appear to be installed\n";

$RECOVER_P_PREAMBLE =
"This restore script is meant to recover a damaged replica from backup. It will
prompt for the basic information it needs to perform its job and then put out
information on the \'restore scenario\' it believes you have in mind based on
the information you have given it. You will then be asked various questions as
the script is going through the restoration process. These question will be in 
the form of prompts and may ask for more information or may simply request 
verification for a particular step it is about to do. You will be told what the
possible responses are as well as any defaults where appropriate. At many of the
prompts a possible response is the string \'quit\'. You may quit at any of these
points. A \'-restart <pathname>\' string will be supplied as the script exits.
You should supply this string before the vob tag argument when you restart the
procedure on the same replica. If you choose to quit at any point DO NOT in any 
way alter the state of the vob or replica before restarting the script on the
same vob."; 


$RECOVER_P_IS_RESTART =
"A previously saved state has been found in the directory
    %s
Restarting a previous restore while ignoring its previous state is not
allowed. This script will use the previous state if the restoration is
to continue. If you wish to quit at this time you may do so. The previous
state will be left intact. 
Do you want to continue at this time? ";

$RECOVER_E_NO_SNAP_DATA =
"Could not open snap data file %s\n";

$RECOVER_E_NO_RECOVER_STATE_FILE = 
"Missing state file %s\n";

$RECOVER_E_HAS_SNAP_ATTRIBUTE =
"The snapshot status of this replica could not be determined.  If you are in 
fact recovering this replica because it was broken this is probably OK. If 
on the other hand you are testing your backups and this replica is actually 
active in some other region it may be a problem. If the snap path is
visible on this host this test recover replica\'s database will be
snapped overwriting the real snap for the replica. If this is the case
you should quit now, determine the cause of the failure and remove the 
snap parameter attribute on this replica with the vob_snapshot_setup script.
Do you wish to quit now? ";

$RECOVER_P_VERIFY_REMOVE_SNAP = 
"This vob is replicated. Removal of the snap parameter attribute will
cause the database to be updated thus breaking any future sync replicas
with this replica. Please verify that this is a backup test and it is
ok to modify the database for this replica. 
Is this a backup test? ";

$RECOVER_P_HAS_SNAP_ATTRIBUTE =
"This replica has a snapshot parameter attribute scheduling it for being
snapped. If you are in fact recovering this replica because it was broken
this is probably ok. If on the other hand you are testing your backups
and this replica is actually active in some other region it may be a
problem. If the snap path
    %s
is visible on this host this test recover replica\'s database will be
snapped overwriting the real snap for the replica. This can be prevented
by removing the snap parameter attribute for this replica at this time.
You should answer yes to this prompt if this recover is a test of backups
otherwise answer no.
Do you want this script to remove the snap parameter attribute ?";

$RECOVER_E_RM_SNAP_ATTRIBUTE_FAILED =
"The removal of the snapshot attribute failed. You should determine the
cause of failure and remove the attribute. Refer to the vob_snapshot_setup
man page. You may restart this script later. The path to give with the
-restart argument when the script is restarted is given below.";

$RECOVER_H_RESTORED_DATA =
"The storage path must be a full local pathname for the directory that either 
does contain or will contain the data restored from backup media. If it has  
not been  restored to disk yet you will be asked to  restore it when it is 
needed. The path  may specify a location which is different from the final
target storage location for the replica. If the restored data will, for example,
end up in the previously registered storage location which is currently being
accessed read only, the path will specify a temporary location where some 
processing will be performed on it. There is no default.\n";

$RECOVER_N_SIGNAL_RECEIVED = 
"\nReceived SIG%s. Calling cleanup routine....\n";

$RECOVER_P_FINISH_CLEANUP =
"Received signal SIG%s while cleaning up. Not finishing cleanup could
leave the vob in a vulnerable state. Do you want to allow the cleanup
to finish?";
    
$RECOVER_H_SAVE_RGY_INFO =
"The vob is about to be unregistered and the script is currently unable
to replace the current settings when it is re-registered. The only things 
that will be given to the cleartool register command when it is re-registered 
are the current tag and the appropriate storage directory. The current 
settings as obtained from
    \'cleartool lsvob -long -region \'*\' -storage <vob_storage> \'
will be saved in 
    %s
You may quit for now if you want to alter the current registry settings
before continuing.\n";

$RECOVER_P_SAVE_RGY_INFO =
"The full current registry settings will be saved in the file
    %s
Press return to continue.";

$RECOVER_N_SAVE_RGY_TAG =
"\n\nThe attempt to save information on all tags for in all regions for 
storage directory
    %s
failed. The script will now try to save information for tag
    %s
in the current hosts registry and region....";

$RECOVER_P_SAVE_RGY_TAG_OK = 
"Attempt succeeded. 
Information on other tags in other regions for this storage directory will
be lost. You may quit for now to attempt to save more information or continue.";

$RECOVER_P_SAVE_RGY_TAG_FAILED = 
"Attempt failed. 
You may quit at this time to attempt to save the current setting
or continue. If you continue only the vob tag and storage path will be
given the re-register command.";

$RECOVER_P_RESTORED_DATA =
"Please specify the full path for the replica storage that was either 
restored from backup media or will be during this restoration process.
Valid responses are (Full path, quit or help)
There is no default response: ";

$RECOVER_H_VOB_STORAGE_IS_REGISTERED   =  
"You must specify a full local pathname for the storage directory to 
contain the replica\'s data when the restoration process is complete.
The default is the currently registered storage path but a different 
one may be chosen.";

$RECOVER_H_VOB_STORAGE_IS_NOT_REGISTERED =
"You must specify a full local pathname for the storage directory to 
contain the replica\'s data when the restoration process is complete. 
Since there is no currently registered local storage on this host for 
this replica there is no default response.\n";

$RECOVER_P_VOB_STORAGE_IS_REGISTERED =
"Specify the full path for target storage directory to contain the replica. 
The default is the currently registered local path:
    %s
Full path, help or quit: ";

$RECOVER_H_SNAPSHOT =
"If you wish this script will merge a snapped database for this replica
which was snapped either prior to or after the replica containers were
backed up. Any required synchronization of the database and containers
will be done by invoking checkvob. You may want to refer to the vob_snapshot
and vob_snapshot_setup manual pages for further information regarding 
snapshots. You may quit at this time if you are unsure what path to give. 
No action will have been taken by this script if you do so.";
 
$RECOVER_P_SNAPSHOT =
"If a merge of a snapped database with this retrieved backup data is
desired, please specify the full pathname of the snapped database.
(Full path, help, quit, <RETURN> for none): ";
 
$RECOVER_P_VOB_STORAGE_IS_NOT_REGISTERED =
"Specify the target storage path to contain the vob. The vob has no 
currently registered local path on this host. There is no default.
(Full path, quit or help)
There is no default response: ";

$RECOVER_H_IS_RESTORE_IN_PLACE =
"The path given for the directory containing the restored data is the same
as the currently registered storage path for the vob. This script can not tell 
if the data contained in this directory is the damaged data being replaced or
the restored data. If it is the original damaged data you will be asked to
perform the restoration when required.\n"; 

$RECOVER_H_IS_RESTORE_IN_PLACE_UNREG =
"The directory given as the one that either does or will contain the 
restored data exists. Since the replica is not registered on this host 
at this time, this script can not determine if the path for this data is 
the same as that for the original damaged data on this host. If it is,
the script must be informed as to whether the data in this directory
is the restored data or the original damaged date.\n";

$RECOVER_P_IS_RESTORE_IN_PLACE =
"Is the data currently contained in the directory
    %s
the data restored from backup media.";

$RECOVER_P_WARN_1_REPLICATED_UNREG =
"Clearcase must have been stopped on this host while the restoration was
in progress. If this was not the case you should redo the restoration before
continuing. Furthermore, if the vob is replicated, this replica should not
have been modified since the restoration. If it is replicated and has been
modified, replica synchronization may be broken. Call your support contact,
DO NOT PROCEED, if this replica has been modified since being restored from 
backup media. 
Do you want to quit at this time?";

$RECOVER_P_WARN_1_REPLICATED =
"Clearcase must have been stopped on this host while the restoration was
in progress. If this was not the case you should redo the restoration before
continuing. Furthermore, since the vob is replicated, this replica should not
have been modified since the restoration. If it has been modified replica 
synchronization is broken. Call your support contact, DO NOT PROCEED, if this 
replica has been modified since being restored from backup media.
Do you want to quit at this time?";

$RECOVER_P_WARN_1_NOT_REPLICATED =
"Clearcase must have been stopped on this host while the restoration was
in progress. If this was not the case you should redo the restoration before
continuing. 
Do you want to quit at this time?";

$RECOVER_P_SNAP_CONFIRM =
"    o The database will be copied from the snapshot directory
           %s
";

$RECOVER_P_ACTIVE_OS_OH_NOTEMP_CONFIRM =
"The original information given and the state of the registry at that time
resulted in the following initial restoration parameters. If this invocation
is a restart the current registry state may be different.

    o The vob is currently registered.
    o The restored vob will be placed in its previously registered location 
          %s
      on its previously registered host
          %s
    o The restoration will be done in place. 
%s
If you wish, you may quite for now  and restart the script at a later time. 
Do you want to proceed?";

$RECOVER_P_ACTIVE_OS_OH_WTEMP_CONFIRM = 
"The original information given and the state of the registry at that time
resulted in the following initial restoration parameters. If this invocation
is a restart the current registry state may be different.

    o The vob is currently registered.
    o The restored vob will be placed in its previously registered location 
          %s
      on its previously registered host
          %s
    o The storage data that has been retrieved from backup media is currently
      in a temporary directory 
          %s
      where some processing will be done prior to moving it to its final 
      destination.
%s
When it becomes necessary to disable the currently active vob, notification 
will be given. If you wish, you may quite for now  and restart the script
at a later time.
Do you want to proceed?";

$RECOVER_P_ACTIVE_NS_OH_CONFIRM      =
"The original information given and the state of the registry at that time
resulted in the following initial restoration parameters. If this invocation
is a restart the current registry state may be different.

    o The vob is currently registered.
    o The restored vob will be placed in a new permanent location 
          %s
      on the its previously registered host.
          %s
%s
When it becomes necessary to disable the currently active vob, notification 
will be given. If you wish, you may quite for now  and restart the script
at a later time.
Do you want to proceed?";

$RECOVER_P_ACTIVE_NS_NH_CONFIRM     =
"The original information given and the state of the registry at that time
resulted in the following initial restoration parameters. If this invocation
is a restart the current registry state may be different.

    o The vob is currently registered.
    o The restored vob will be placed in a new permanent location 
          %s
      on a new host
          %s
%s
When it becomes necessary to disable the currently active vob, notification 
will be given. If you wish, you may quite for now  and restart the script
at a later time.
Do you want to proceed?";

$RECOVER_P_INACTIVE_CONFIRM  =
"The original information given and the state of the registry at that time
resulted in the following initial restoration parameters. If this invocation
is a restart the current registry state may be different.

    o The vob is currently unregistered.
    o The restored vob will be placed in
          %s
      on host
          %s
%s
If you wish, you may quite for now  and restart the script at a later time. 
Do you want to proceed?";

$RECOVER_P_OTHER_UNREGISTERS =
"If the replica is currently registered in other registries it must be 
unregistered from them now. You may quit for now to perform this operation
or press <RETURN> to continue only when it has been completed.";

$RECOVER_H_NEEDS_DBCHECK =
"A dbcheck on the database was not done at the time of the snap. One may
be done now if you wish. The recovery will continue if you elect not to do
a dbcheck at this time. Once the restored vob has been reactivated, this 
script will determine if the snapshot operation was done with the vob unlocked. 
If this is found to be the case, the  script will not make  the vob available 
until the dbcheck is done and the database is found to be consistent.";

$RECOVER_P_NEEDS_DBCHECK =
"A dbcheck was not done during the snapshot. Do you wish to do one now?";

$RECOVER_H_VERIFY_POOLS = 
"All of the recovered pools must be in place at this time. This includes
remote pools associated with this replica. If this has not been completed 
yet it must be done at this time. You may quit at this time and restart the 
script when the pools are in place.\n";

$RECOVER_P_VERIFY_POOLS =
"If there are any remote pools that should be restored now is the time to
restore them. You may quit for now to perform this operation or press.";

$RECOVER_H_REMOVE_OLD_RESTORE_DIR = 
"The snapshot area is being copied into the restored storage area. A 
directory is about to be copied in for which there is a previously
existing copy. This old copy was likely picked up by the system backup
utility when the containers were also picked up. If you don't still
have a copy on backup media you may want to save a copy before continuing.
Proceeding will remove the directory.";

$RECOVER_P_REMOVE_OLD_RESTORE_DIR = 
"The restored storage area contains a copy of a directory being restored
from the snapshot area. 
    %s
Is it ok to remove this copy? ";

$RECOVER_H_REMOVE_OLD_RESTORE_FILE = 
"The snapshot area is being copied into the restored storage area. A 
file is about to be copied in for which there is a previously existing 
copy. This old copy was likely picked up by the system backup utility 
when the containers were also picked up. If you don't still have a copy 
on backup media you may want to save a copy before continuing. Proceeding 
will remove the file.";

$RECOVER_P_REMOVE_OLD_RESTORE_FILE = 
"The restored storage area contains a copy of a file being restored from 
the snapshot area. 
    %s
Is it ok to remove this copy? ";

$RECOVER_P_DBCHECK_REQUIRED =
"The database was backup without the lock being held. This may leave the
database in an inconsistent state. Checkvob can not be run until the 
consistency of the database has been verified. No dbcheck has been done
on this database thus far during this restoration so one must be performed 
prior to continuing.
Do you wish to do the database check at this time?";

$RECOVER_H_CCASE_MADE_UNAVAILABLE =
"All restore activity that may be done while the vob is still active has 
been completed. The currently registered replica storage must now be made 
unavailable. All activity should be stopped prior to proceeding. Upon 
proceeding the replica will be unregistered and all of its tags removed 
from the current host's registry. If the replica is registered in other 
registries it must be unregistered and its tags removed from those registries 
at this time. This script can not perform the unregister operations in other 
registries. If this needs to be done quit now, perform the unregister 
operations and restart at a later time. DO NOT UNREGISTER the replica on 
this host however. Leaving the vob registered in registries other than the
current host's registry will not prevent the recovery from completing, but
users will see access errors when the old storage is moved or removed a short 
time from now.\n";

$RECOVER_P_CCASE_MADE_UNAVAILABLE =
"The vob must now be made unavailable. This script will unregister the vob from
this hosts\'s registry. If it is registered in any other registries you must
unregister it from those registries before continuing. Do not unregister it from
this host's registry. You may quit for now to perform this operation. Press
<RETURN> to continue only when this host's registry is the only registry the
replica remains registered in.";
			   
$RECOVER_H_STOP_START_CCASE_FOR_IN_PLACE =
"Clearcase must be stopped and restarted on this host prior to restoring data 
from backup media to ensure all clients that may be accessing this replica are 
disconnected. The restart will be automatic and immediate.\n";

$RECOVER_H_STOP_START_CCASE_THIS_HOST =
"Clearcase must be stopped and restarted to ensure all clients that may be
access the replica are disconnected. If the old storage still exists it must
be moved aside while clearcase is stooped. This script will instruct you to 
do so if necessary.\n";

$RECOVER_P_STOP_START_CCASE_THIS_HOST =
"Is it all right to shutdown Clearcase on this host?";

$RECOVER_H_STOP_START_CCASE_OTHER_HOST = 
"The stop and restart of Clearcase ensures all clients that may be accessing
the storage are disconnected prior to it being moved aside.\n";

$RECOVER_P_STOP_START_CCASE_OTHER_HOST =
"Clearcase must be stopped on the host that stores the old storage for the 
restored replica. If the old storage still resides on that host it must be
either removed or moved aside. Once this has been done Clearcase may be 
restarted. The old storage is stored at path
    %s
on the host
    %s
You may quit at this time to perform this operation. Press <RETURN> to 
continue only after it has been completed.";

$RECOVER_P_STOP_START_CCASE_OTHER_HOST_UNREG =
"Clearcase must be stopped on the host that stores the old storage for the 
restored replica. If the old storage still resides on that host it must be
either removed or moved aside. Once this has been done Clearcase may be 
restarted. You may quit at this time to perform this operation. Press 
<RETURN> to continue only after it has been completed.";

$RECOVER_H_STOP_START_CCASE_UNREG =
"Clearcase must be stopped and restarted on the host on which the vob
previously resided. This must be done to ensure all clients that may
be referencing the vob are disconnected. The fact that the vob is
currently unregistered does not assure this. This script can not
determine what host the vob last ran on. If that host was this host
the script will perform the stop/start, otherwise you will be instructed
to do so.\n";

$RECOVER_P_WAS_VOB_ON_THIS_HOST =
"Did this replica's storage previously reside on this host?";

$RECOVER_H_RESTORE_STORAGE =
"All processing that can be done prior to placing the restored data in place
has been performed. The data restoration must be done before continuing. If
you quit now the script will pick up at this point when it is restarted.\n";

$RECOVER_P_RESTORE_STORAGE =
"The data restoration from backup media to the target storage directory 
    %s
must be done now. You may quit for now to perform the restoration. Press
<RETURN> to continue only when it has been completed.";

$RECOVER_H_MOVE_OR_REMOVE_OLD_STORAGE = 
"The previous storage area should now be moved aside. This must be 
done even if you are recovering the replica to a new location. If the
replica is being restored to a new location, on the same disk partition 
as the previous location you may have to remove the old to make space 
for the new. In this case you may want to back it up if it isn't 
already. You must either move the previous storage aside or remove it. 
If you elect to quit for now, the script will attempt to begin where it left 
off when restarted.\n";

$RECOVER_P_MOVE_OLD_LOCAL_STORAGE = 
"You must either move or remove the previous storage directory 
    %s
now. You may quit for now to perform this operation. Press <RETURN>
to continue only when it has been completed.";

$RECOVER_P_STORE_ALREADY_EXISTS =
"An entity with the same path as the target destination already exists. 
The temporary data can not me moved into its final location until this 
entity is either moved or removed. Please move or remove
    %s
now. You may quit for now to perform this operation. Press <RETURN>
to continue only when it has been completed."; 

$RECOVER_P_GET_RESTORED_DATA =
"The restored data must now be retrieved from backup media and placed in
    %s
You may quit quit to restore the data now or press <RETURN> to continue
when the restoration has been completed.";

$RECOVER_H_MOVE_TEMP_TO_DEST = 
"All processing on the temporary storage is complete and the vob's previous
storage has been moved aside or removed. The temporary storage must now be
copied into its final destination. If you elect to quit for now, the script
will attempt to begin where it left off when restarted.\n";

$RECOVER_P_MOVE_TEMP_TO_DEST =  
"The temporary storage
    %s
must now be moved into its final destination
    %s
Do you want to proceed? ";

$RECOVER_H_RUN_CHECKVOB =
"Checkvob examines the vob storage containers and database for possible
problems. One primary source of problems that checkvob looks at are
inconsistencies due to a snapped database being restored to a vob whose
containers are somewhat younger that the snapped database. Checkvob may 
be invoked in \'check only\' or fixed mode. The former examines and reports
problems but does no repair work. Fix mode does repair. You will be given
the opportunity to run either or both of these if you wish. If you elect 
to quit for now, the script will attempt to begin where it left off when 
restarted.\n";

$RECOVER_P_RUN_CHECKVOB =
"Do you want to have checkvob examine the vob for possible problems?";

$RECOVER_P_CHECKVOB_RUN_SUMMARY =
"Would you like to have checkvob run in \'check only\' mode?";

$RECOVER_H_CHECKVOB_RUN_SUMMARY =
"Checkvob may be run in  \'check only\' mode prior to having it repair any 
damage. In this mode it will generate a set of rather extensive summary files 
containing information on what kind of problems are present. These files
are placed in a log directory whose name is generated by this script. When
the summary pass completes, this script will report the number of corrupt
or missing containers and the number of misprotected containers that are
present. On large vobs the summary may take a long time to complete. Its
scope may be limited to some pool types rather than all to lessen the impact.
You will be given the opportunity to select pool types. If you elect to quit 
for now, the script will attempt to begin where it left off when restarted.\n";

$RECOVER_H_CHECKVOB_REUSE_SUMMARY = 
"A previous invocation ran checkvob in \'check only\' mode and the output logs 
are still available. You may examine the previous logs now if you wish. If upon 
examination you want to rerun checkvob in \'check only\' mode. simply quit at the 
next prompt. The script will bring you back to this point and you may then elect 
to skip the previous summary and generate a new one. You might want to do this 
if, for example, The previous run did not examine a pool type you are now interested 
in having it look at.\n";

$RECOVER_H_NUSERS_OK =
"This is particularly important if this replica is part of a replicated vob.
replicated. Updating the retired backup before replica syncronization takes 
place may break future replication for the vob. If the vob is replicated and 
this replica get updated by anyone other that this script before the replica
synchronization takes place call you support contact immediately.\n";

$RECOVER_P_NUSERS_OK =
"While running checkvob the the vob will be locked for all but user
    %s
The vob will be open to possible modifications by anyone running as 
this user. This MUST NOT be allowed to happen. If you need to make
arrangements to ensure that no one with this identity will attempt to
modify this vob before proceeding you may quit for now. Press <RETURN>
if you are satisfied that no such modifications will occur.";

$RECOVER_P_CHECKVOB_REUSE_SUMMARY =
"This script invocation is a restart and a checkvob summary file
    %s
still exists. Would you like to see its formatted contents.?";

$RECOVER_P_CHECKVOB_SUMMARY_GO =
"Checkvob will now be run in \'check only\' mode. This may take a while on 
large vobs. Checkvob will generate logs in the directory
    %s
It will emit rather verbose information to standard output as it runs
All of this information is also saved in the log directory for later
viewing. You may be prompted for information as well. You may quit for
now or press <RETURN> to continue.";

$RECOVER_CHECKVOB_SUMMARY_RESULTS =
"The following is a summary of the checkvob summary run whose log directory 
    %s

";

$RECOVER_MORE_SUMMARY_DETAILS =
"More details may be found in the log files in the above directory\n";

$RECOVER_H_CHECKVOB_RUN_FIX = 
"If any inconsistencies were detected by a checkvob summary pass they
should be fixed before continuing. It a summary was not generated, the 
state of the vob is unknown. It is particularly important to run a summary
and/or fix pass if the restoration includes a snapshot recovery. In this 
case the containers and database may be somewhat out of sync. Checkvob
will take care of these problems. If you elect to not run either a summary
and/or fix at all, the vob will be left locked. Check vob will leave logs
in a log directory is generated by this script. When the repair pass completes
you will be given a short summary of things that were fixed which resulted
in version or data loss. You will also be informed of corruption or protection
problems that checkvob was unable to repair. You should contact your support
representative to resolve the later. If you elect to quit for now, the script 
will attempt to begin where it left off when restarted.\n";

$RECOVER_P_CHECKVOB_RUN_FIX =
"Do you want to have checkvob run in repair mode? ";

$RECOVER_H_CHECKVOB_REUSE_FIX = 
"This script remembers some things across invocations until the the vob has
been marked as well. A previous invocation ran checkvob in repair mode and
the output logs are still available. You may examine the previous logs now if
you wish. If upon examination you want to rerun checkvob in repair mode, 
simply quit at the next prompt. The script will bring you back to this point
and you may then elect to skip the previous summary and generate a new one. 
You might want to do this if, for example, the previous run did not repair
a pool type you are now interested in repairing.\n";

$RECOVER_P_CHECKVOB_REUSE_FIX =
"This script invocation is a restart and a checkvob fix summary
    %s
still exists. Would you like to see its formatted contents.?";

$RECOVER_P_CHECKVOB_FIX_GO =
"Checkvob will now be run in fix mode. This may take a while on 
large vobs. Checkvob will generate logs in the directory
    %s
It will also emit rather verbose information to standard output as it
runs. All of this information is also saved in the log directory for
later viewing. You may be prompted for information as well. 
Do you want to proceed? ";

$RECOVER_P_GET_CHECKVOB_TAG =
"Checkvob expects to be run in a view context. If you are not currently 
in a view, checkvob will accept a view tag argument. Supply one now if
you are not in a view.
view_tag: ";

$RECOVER_E_CHECKVOB_SUMMARY_FAILED =
"The checkvob summary command failed. Determine what caused the failure
and restart the script. It will attempt to pick up where it left off.\n";

$RECOVER_E_SUMMARY_OPEN_FAILED = 
"Could not open the checkvob summary file 
    %s \n";

$RECOVER_PARSE_CHECKVOB_SUMMARY_FAILED =
"Parsing of the checkvob summary file 
    %s
failed\n";

$RECOVER_PARSE_CHECKVOB_FIX_FAILED =
"Parsing of the checkvob post fix summary file
    %s

failed and errors still remain in the VOB pools.\n";

$RECOVER_PARSE_CHECKVOB_FIX_FAILED_NO_DIR =
"Parsing of the checkvob post fix summary file
    %s

failed, as the directory does not exist.  This is due to the checkvob
command never writing the log file and more than likely terminated
either by user choice or prematurely.  Errors still remain in the VOB pools.\n";

$RECOVER_CHECKVOB_FIX_LEFT_ERRORS =
"Checkvob run in fix mode was unable to repair all damage. If the summary
file
    %s
exists examine it to try to determine the nature of the failure. If the file
does not exist try to determine why checkvob could not begin to perform its
task. When the cause of the problem has been detected and repaired either run
checkvob by hand or restart this script with the -restart path given below.\n";

$RECOVER_CHECKVOB_NOT_RUN_REPLICATED =
"\n\nCheckvob was not run after recover. This replica is part of a replicated 
vob hence checkvob MUST complete prior to replica synchronization. Replica
synchronization MUST then be performed prior to any modifications of the
replica. You may either run checkvob by hand or restart this script with
the -restart path given below.\n";
 
$RECOVER_CHECKVOB_NOT_RUN =
"\n\nCheckvob was not run . It is STRONGLY ADVISED that you either run
checkvob by hand or rerun this script with the -restart path given below.\n";
 
$RECOVER_CHECKVOB_CLEAN_REPLICATED =
"\n\nCheckvob has either detected no problems or has repaired what it did 
detect. This replica is part of a replicated vob hence any necessary replica
synchronization MUST BE PERFORMED before any modifications are made to
this replica.\n";

$RECOVER_CHECKVOB_CLEAN =
"\n\nCheckvob has either detected no problems or has repaired what it did 
detect. This is a non replicated vob so no further action should be required.\n";

$RECOVER_E_INIT_C_CNT =
"%d %s containers are either missing or corrupt\n";
$RECOVER_E_INIT_M_CNT =
"%d %s containers are misprotected\n";

$RECOVER_FN_C_OBJS =
"%d %s containers remain either missing or corrupt\n";
$RECOVER_FN_M_OBJS =
"%d %s containers remain misprotected\n";
$RECOVER_FX_LO_SRC =
"%d source elements experienced loss of version data\n";
$RECOVER_FX_LO_DO =
"%d rmdo operations were done\n";
$RECOVER_FX_LI_SRC =
"%d source versions were lost\n";
$RECOVER_FX_LI_DO =
"%d derived objects were lost\n";
$RECOVER_CLEARTXT_NOT_CHECKED =
"Clear text containers were not checked.\n";

$RECOVER_E_CHECKVOB_FIX_FAILED =
"Checkvob run in fix mode failed. If the summary file
    %s
exists examine it to try to determine the nature of the failure. If the file
does not exist try to determine why checkvob could not begin to perform its
task. When the cause of the problem has been detected and repaired either run
checkvob by hand or restart this script with the -restart path given below.\n";
 
$RECOVER_E_CHECKVOB_FIX_FAILED_REPLICATED =
"Checkvob run in fix mode failed. This replica is part of a replicated vob
and checkvob MUST complete prior to replica synchronization. If the summary
file
    %s
exists examine it to try to determine the nature of the failure. If the file
does not exist try to determine why checkvob could not begin to perform its
task. When the cause of the problem has been detected and repaired either run
checkvob by hand or restart this script with the -restart path given below.\n";

$RECOVER_P_CHECKVOB_CHECK_SOURCE =
"Source pools?";

$RECOVER_P_CHECKVOB_CHECK_DERIVED =
"Derived object pools?";

$RECOVER_P_CHECKVOB_CHECK_CLEARTXT =
"Clear text pools?";

$RECOVER_H_GET_TAG =
"Checkvob can print human meaningful container information if it
has a view context from which to run. Please supply the name of
a view.";

$RECOVER_STATE_FILE_LOCATION =
"\nThis script has stored its current state. The script must be restarted
with the following restart flag option and argument.

     -restart %s\n";

#
#   The name of this program. It gets passed to mdep_error.
#
$RECOVER_PROGRAM                = "vob_restore";

#
#   The scenario definitions. See the comment before &recover_set_scenario
#   for meanings.
#   
$RECOVER_SC_UP_OS_OH_W_TEMP    = 1;
$RECOVER_SC_UP_OS_OH_NO_TEMP   = 2;
$RECOVER_SC_UP_NS_OH           = 3;
$RECOVER_SC_UP_NS_NH           = 4;
$RECOVER_SC_DOWN               = 5;
$RECOVER_SC_IS_UNKNOWN         = 6;

#
#    The associative list %recover_state indexed by state strings tells
#    us what was done on previous runs if this is a restart. It is filled
#    in by &recover_open_state. &recover_st_log logs a state during a run
#    and &recover_st_chk check the current state value for the state 
#    string passed in. Most state values are just on or off but some 
#    have values. The associative list is 
#
#      %recover_state
#    The state strings are listed here. &recover_print_current_state 
#    will print out all the current values.
#
#     recover_st_scenario            # The chosen scenario
#     recover_st_dbcheck_done        # Dbcheck is done
#     recover_st_restore_snap_done   # Snapshot restore complete
#     recover_st_data_protected      # Data base is protected
#     recover_st_dbcheck_needed      # A dbcheck is required
#
#

    

# The following concerns the checkvob protocols....
#
# There are three pool kinds - source, cleartext, and derived. Each
# kind has an associative list indexed by field names. The field names
# are 
#     Pool="<poolname>"           Quotes will always be present and are not
#                                 part of the name.  A nil pool name ('""')
#                                 represents the aggregate status of all
#                                 processed "poolkind" pools.
#     InitialNdataObjs=<n>        The initial number of 'ndata' objects.
#                                 ('ndata' in this context means: having
#                                  missing or corrupt containers)
#     InitialMisprotectedObjs=<n> The initial number of misprotected objects
#     FinalNdataObjs=<n>          The final number of 'ndata' objects.
#     FinalMisprotectedObjs=<n>   The final number of misprotected objects.
#     FixedDataLossObjs=<n>       The number of objects that were fixed but
#                                 incurred a data loss.
#                                 For a source pool this is the number
#                                 of fixed elements that experienced a
#                                 loss of version data.
#                                 For a DO pool this means the number
#                                 of 'rmdo' operations that were done.
#     FixedDataLossItems=<n>      The number of 'items' with lost data.
#                                 For a source pool this means the number
#                                of versions that were lost (hence made
#                                 'ndata' via "rmver -data").
#                                 For a DO pool this means the number of DOs
#                                 that were lost (hence removed).
# PoolKind is also a valid field but we don't keep it since the list name
# includes it. 
# 
# The format of the .summary files is...
#
#   Each line will be a 'record'.
#   Each record type will be identified by an identifier.
#   record types with multiple fields will have
#       <fieldName>=<fieldValue> constructs.
#       Fields are separated by whitespace.
#       No field ordering constraints are imposed.
#

#
# The associative lists are
#     %recover_source_pool_record
#     %recover_cleartext_pool_record
#     %recover_derived_pool_record
# The associative field indexes are the same as the names above.
#
$RECOVER_PL_POOL_KIND                  = 'PoolKind';
$RECOVER_PL_POOL_NAME                  = 'Pool';
$RECOVER_PL_INIT_NDATA_OBJS            = 'InitialNdataObjs';
$RECOVER_PL_INIT_MISPROT_OBJS          = 'InitialMisprotectedObjs';
$RECOVER_PL_FIN_NDATA_OBJS             = 'FinalNdataObjs';
$RECOVER_PL_FIN_MISPROT_OBJS           = 'FinalMisprotectedObjs';
$RECOVER_PL_FIX_DATA_LOSS_OBJ          = 'FixedDataLossObjs';
$RECOVER_PL_FIX_DATA_LOSS_ITEMS        = 'FixedDataLossItems';

# 
# The presence of a Pool Kind is indicated by...
#
$RECOVER_PL_IS_PRESENT                 = 1;
$RECOVER_PL_NOT_PRESENT                = 0;
$recover_source_pool_present             = $RECOVER_PL_NOT_PRESENT;
$recover_cleartext_pool_present          = $RECOVER_PL_NOT_PRESENT;
$recover_derived_pool_present            = $RECOVER_PL_NOT_PRESENT;

#
# Before a pool is marked present, all of its associative values
# are set to either "" or -1, whichever is appropriate.
#

# 
# For parsing purposes keep a list of the known field name
#
push(@recover_known_pool_fields,$RECOVER_PL_POOL_KIND);
push(@recover_known_pool_fields,$RECOVER_PL_POOL_NAME);
push(@recover_known_pool_fields,$RECOVER_PL_INIT_NDATA_OBJS);
push(@recover_known_pool_fields,$RECOVER_PL_INIT_MISPROT_OBJS);
push(@recover_known_pool_fields,$RECOVER_PL_FIN_NDATA_OBJS);
push(@recover_known_pool_fields,$RECOVER_PL_FIN_MISPROT_OBJS);
push(@recover_known_pool_fields,$RECOVER_PL_FIX_DATA_LOSS_OBJ);
push(@recover_known_pool_fields,$RECOVER_PL_FIX_DATA_LOSS_ITEMS );

#
# Be paranoid. Initialize the globals. Good programming anyway.
#

$recover_arg_vob_tag           = "";
$recover_arg_snap_storage      = "";
$recover_arg_dest_storage      = "";
$recover_arg_temp_storage      = "";
$recover_registered            = 0;
$recover_rgy_storage           = "";
$recover_rgy_replica_uuid      = "";
$recover_rgy_family_uuid       = "";
$recover_rgy_host              = "";
$recover_host_name             = "";
$recover_scenario              = $RECOVER_SC_IS_UNKNOWN;
#recover_snapath_complete      = 0;
$recover_restart               = 0;
$recover_replicated           = 0;


$RECOVER_USAGE = 
"Usage: vob_restore [-restart <previous-state-path>] vob-tag\n";

    $RECOVER_IS_WRITABLE = "writable";
    $RECOVER_IS_READABLE = "readable";
    $RECOVER_IS_EXECUTABLE = "executable\n";



sub mdep_is_clearcase_up {
    
    local(@cmdbuf); 
    local($command);
    local($status);

    if($MDEP_MACHINE eq 'NT') {
	$command = $MDEP_CCASE_RUNNING;
	$status = &mdep_command_s(*command,*cmdbuf);
	if($status) {
	    
	    return -1;
	}
	if(grep(/Atria Location Broker/,@cmdbuf)) {
	    
	    return 1;
	}
	else {
	    
	    return 0;
	}
    }
    else {
	$command = $MDEP_CCASE_RUNNING;
	$status = &mdep_command_s(*command,*cmdbuf);
	if($status) {
	    
	    return -1;
	}
	if(grep(/albd/,@cmdbuf)) {
	    
	    return 1;
	}
	else {
	    
	    return 0;
	}
    }
}

sub Rmtag {

    local($vobtag) = @_[0];       # Vob tag
    local($status);               # Return status
    local(@cmdbuf);               # Return buffer for mdep_command
    local($line);                 # Line buffer for parsing
    local($ret_status);           # Return status

  
CLEANUP_RMTAG: { # Always return at end of this block.

    if($vobtag eq "") {

	$ret_status = $RTN_VOB_TAG_NULL;
	last CLEANUP_RMTAG;
    }
    if($CLEARCASE_LIGHT) {
        $status = system("$MDEP_CLEARTOOL rmtag -vob $vobtag");
    } else {
        $status = system("$MDEP_CLEARTOOL rmtag -vob -all $vobtag");
    }
    
    if($status) {

	$ret_status =  $RTN_RMTAG_FAILED;
	last CLEANUP_RMTAG;

    }

    $ret_status = $RTN_RMTAG_OK;

  } # CLEANUP_RMTAG Always return here

    return $ret_status;
}

sub mdep_stop_clearcase_nt {

    local($status);        # Return status
    local($tmpname);
    local($line);

    $tmpname = &mdep_get_tmpname();
    $status = system("net stop albd > $tmpname");
    $status = $status + system("net stop lockmgr > $tmpname");
    print "Waiting for servers to stop\n";
    sleep(30);

    if($status) {

	close(TMPFILE);
	&mdep_rmfile($tmpname);
	return $RTN_CCASE_STOP_FAILED;
    }
    close(TMPFILE);
    &mdep_rmfile($tmpname);
    return $RTN_CCASE_STOP_OK;
}  



sub mdep_start_clearcase_nt {

    local($status);        # Return status
    local($tmpname);
    local($line);
    local($ret_status);

CLEANUP_START_CLEARCASE: {

    $tmpname = &mdep_get_tmpname();
    $status = system("net start albd > $tmpname");
    $status = $status + system("net start lockmgr > $tmpname");

    if($status) {

	close(TMPFILE);
	&mdep_rmfile($tmpname);
	$ret_status =  $RTN_CCASE_START_FAILED;
    }

    $ret_status = $RTN_CCASE_START_OK;

} CLEANUP_START_CLEARCASE:

    close(TMPFILE);
    &mdep_rmfile($tmpname);
    return $ret_status;
}  


#
# recover_set_state
# Inputs:
#   $state 
#      state string
# Outputs:
#   None
# Returns:
#   1 If it got set
#   0 otherwise
# References:
#   %recover_current_state
# Modifies:
#   None
# Calls:
#   None
# Description:
#   Set the state string IF it hasn't already been done.
#
sub recover_set_state {

    local($state) = @_[0];       # State string
    local($sname);               # State name
    local($svalue);              # State value
    local($cvalue);              # Current state value
    local($tstate);              # Temporary state string holder
    local($ret_status);          # Return status

    &recover_set_cur_function("recover_set_state");

CLEANUP_SET_STATE: {

    #
    # The string may include name and value. Get the name.
    #

    if($state =~ /(\w+)=(.*)/) {

	$sname = $1;
	$svalue = $2;
    }
    else {
	
	#
	# Nope. Give it a default value of 1.
	#
	$sname = $state;
	$svalue = 1;
    }

    #
    # Get the current value.
    #
    $cvalue = $recover_current_state{$sname};

    if($cvalue && ($cvalue ne "REMOVED")) {

	#
	# A valid value already exists for this state. Don't
        # do it again.
	#
	$ret_status = 0;
	last CLEANUP_SET_STATE;
    }

    #
    # Not yet. Set it.
    #
    $recover_current_state{$sname} = $svalue;
    $ret_status = 1;

  } # CLEANUP_SET_STATE

    &recover_restore_cur_function();
    return $ret_status;
}

#
# recover_unset_state
# Inputs:
#   $state 
#      state string
# Outputs:
#   None
# Returns:
#   None
# References:
#   %recover_current_state
# Modifies:
#   None
# Calls:
#   None
# Description:
#   Remove any state string with the same name as $state from the
#   @recover_current_state list.
#
sub recover_unset_state {

    local($state) = @_[0];       # State string
    local($sname);               # The state name
    local($ret_status);          # Return status

    &recover_set_cur_function("recover_unset_state");
    #
    # The string may include name and value. Get the name.
    #
    if($state =~ /(\w+)=(.*)/) {

	$sname = $1;
	$svalue = $2;
    }
    else {
	
	#
	# Nope. Give it a default value of 1.
	#
	$sname = $state;
	$svalue = 1;
    }
    $recover_current_state{$state} = 'REMOVED';

    &recover_restore_cur_function();

    return $ret_status;

}



sub recover_print_current_state {

    local($banner) = @_[0];
    local($state);

    print "$banner\n";
    foreach $state (keys(%recover_current_state)) {

	print "$state: $recover_current_state{$state}\n";
    }
}

sub recover_write_state {

    local($pdir);
    local($state);
    local($tstate);
    local($dstate);
    local($cstate);
    local($ret_status);

    &recover_set_cur_function("recover_write_state");

CLEANUP_WRITE_STATE: { # Always return at end of this block.



    $cstate = $recover_state_dir.$MDEP_SEP.$recover_state_file;

    #
    # Check the parens directory for permission.
    #
    $pdir = &mdep_dirname($cstate);
    if(! -w $pdir) {
 
        $ret_status =  $RTN_NOT_OK;
        last CLEANUP_WRITE_STATE;
    }
    #
    # Open it for write.
    #

    unless(open(RECOVER_STATE_FILE,">$cstate")) {
 
        $ret_status =  $RTN_NOT_OK;
        last CLEANUP_WRITE_STATE;
    }

    foreach $state (keys(%recover_current_state)) {
	if($recover_current_state{$state} ne "REMOVED") {

	    print RECOVER_STATE_FILE "$state=$recover_current_state{$state}\n";
	}
    }

  } # CLEANUP_WRITE_STATE Always return here

    &recover_restore_cur_function();
    close(RECOVER_STATE_FILE);
}

#
# recover_error()
# Inputs:
#   $errstr 
#      The error string to put out.
# Outputs:
#   None
# Returns:
#   None
# Modifies:
#   None
# Calls:
#   &mdep_error()
# Description:
#   A wrapper for the call to mdep_error. It adds the program name and
#   error string.
#
sub recover_error {

    local($errstr) = @_[0];   # The string to emit.
    local($curfunc);          # The currently executing function

    $curfunc = pop(@recover_cur_function);
    &mdep_error($RECOVER_PROGRAM,$curfunc,$MDEP_ERROR,$errstr);
}


#
# recover_set_cur_function
# Inputs: 
#   $function 
#     String containing the function being entered
# Outputs: 
#   None
# References: 
#   None
# Modifies:
#   $recover_cur_function
# Returns:
#   None
# Calls: 
# Description:
#   Pushes the function string for the function being entered onto the
#   top of @recover_cur_function;
#   
#
sub recover_set_cur_function {

    local($function) = @_[0];       # Entered function

    push(@recover_cur_function,$function);
}

#
# recover_restore_cur_function
# Inputs: 
#   $function 
#     String containing the function being exited
# Outputs: 
#   None
# References: 
#   None
# Modifies:
#   @recover_cur_function
# Returns:
#   None
# Calls: 
# Description:
#   Pops the currently executing function off @recover_cur_function.
#
sub recover_restore_cur_function {

    pop(@recover_cur_function);
}

#
# recover_check_clearcase()
# Inputs: 
#   None
# Outputs: 
#   None
# References: 
#   None
# Modifies:
#   None
# Returns:
#   None
# Calls: 
#   &recover_error()
# Description:
#   Check to see if $MDEP_CLEARTOOL is installed and executable.
#   Fatal error if it is not.
#
sub recover_check_clearcase {

    local($ret_status);        # Status return at end of CLEANUP block
    

    &recover_set_cur_function("recover_check_clearcase");
    $ret_status = $RTN_NO_STATUS_SET; 

CLEANUP_CHECK_CLEARCASE: { # Always return at end of this block.

    # 
    # If cleartool exists and is executable, assume clearcase
    # is installed.
    #
    if( ! -x $MDEP_CLEARTOOL ) {

	&recover_error($RECOVER_E_CLEARCASE_NOT_INSTALLED);
	&recover_cleanup();
    }

    $status = &mdep_is_clearcase_up();
    if($status < 0) {
	
	&recover_error($RECOVER_E_CCASE_UP_CHECK);
	&recover_cleanup();
    }
    
    if(!$status) {

	&recover_error($RECOVER_E_CCASE_MUST_BE_UP);
	&recover_cleanup();
    }

  } # CLEANUP_CHECK_CLEARCASE Always return here

  &recover_restore_cur_function();
  return $ret_status;   

}

#
# recover_parse_arguments()
# Inputs: 
#   @ARGV untouched.
# Outputs: 
#   None
# References: 
#   None
# Modifies
#   $recover_arg_vob_tag
#   $recover_restart
# Called:
#   recover_error()
# Description:
#   Parse the argument list setting the arg globals. 
#   Fatal on parse errors.
#   Fatal error if the required  recover_arg_vob_tag argument wasn't 
#   passed.
#
sub recover_parse_arguments {

    local($arg);               # Temp arg holder
    local($replay);            # Reply buffer when communicating with &mdep_prompt
    local($ret_status);        # Status return at end of CLEANUP block

    &recover_set_cur_function("recover_parse_arguments");
    $ret_status = $RTN_NO_STATUS_SET; 

CLEANUP_PARSE_ARGUMENTS: { # Always return at end of this block.
    #
    # Parse them in. $arg should always get set to a switch starting
    # with a '-'. Any arguments accompanying the switch will get consumed
    # in before arg gets set again. Check for '-' first to detect 
    # parsing errors.
    #
    $_ = shift;
    if(/^-/) {
	if($_ = "-restart") {
	    $recover_state_dir = shift;
	    $_ = shift;
	}
	else {
	    print "Error: Unexpected argument \'$_\'\n";
	    print $RECOVER_USAGE;
	    exit(1);
	}
    }

    if(!$_) { 
	print "Error: A vob-tag argument is required\n";
	print $RECOVER_USAGE;
	exit(1);
    }

    if(/^-/) {
	print "Error: Unexpected argument \'$_\'\n";
	print $RECOVER_USAGE;
	exit(1);
    }

    $recover_arg_vob_tag = $_;

    if(scalar @_) {
	print "Error: Unexpected arguments: ";
	while($_ = shift) {
	    print "\'$_\' ";
	}
	print "\n";
	print "$RECOVER_USAGE";
	exit(1);
    }

    $recover_state_file = "recover_state".'_'.&mdep_basename($recover_arg_vob_tag);

    if($recover_state_dir) {

	if(! -e $recover_state_dir.$MDEP_SEP.$recover_state_file) {
	    $fstr = sprintf($RECOVER_E_NO_RECOVER_STATE_FILE,
			    $recover_state_dir.$MDEP_SEP.$recover_state_file);
	    &recover_error($fstr);
	    exit(1);
	}
	if(!&recover_open_state_file()) {
	    &recover_error("Could not open previous state file\n");
	    exit(1);
	}
	&recover_set_inputs();
	$recover_restart = 1;
	$recover_do_save_state = 1;
    }
	
  } # CLEANUP_PARSE_ARGUMENTS Always return here
  &recover_restore_cur_function();
  return $ret_status;   

}  

# 
# recover_open_state_file
# Inputs:
#   $state_file
#     Path to the existing trial file;
# Outouts:
#   None

#   $RECOVER_
# References:
#   None
# Modifies:
#   None
# Calls:
#   &recover_error
# Description:
#
sub recover_open_state_file {
 
    local($tpath) = "";           # Path if its in the temp restore area
    local($dpath);                # Path if its in the dest restore area
    local($spath) = "";           # Where it really is,
    local($pdir);                 # Parent directory
    local($oldfh);                # Current file handle
    local($nostate) = 1;          # No previous state.
    local($state);                # Holds read state
    local($ret_status) = "";        # Where the state file was found
    local($state_path);

 
    &recover_set_cur_function("recover_open_state_file");
    
CLEANUP_OPEN_STATE_FILE: { # Always return at end of this block.

    $state_path = $recover_state_dir.$MDEP_SEP.$recover_state_file;

    #
    # One is there. Open it, read it in and close it. Then
    # remove it.
    #
    unless(open(RECOVER_STATE_FILE,"$state_path")) {
	
	$ret_status = 0;
	last CLEANUP_OPEN_STATE_FILE;
    }
    
    #
    # Read it in setting various things.
    #
    foreach $state (<RECOVER_STATE_FILE>) {

	chop $state;
	&recover_set_state($state);
	
    }

    close(RECOVER_STATE_FILE);
    &mdep_rmfile($state_path);
    
    $ret_status = 1;
    
} # CLEANUP_OPEN_STATE_FILE
 
    &recover_restore_cur_function();
    
    return $ret_status;
 
}

# 
# recover_st_chk
# Inputs:
#   $state
#     state string.
# Outouts:
#   None
# Returns:
#   1 If the state associated with $state is true
#   0 Otherwise.
# References:
#   None
# Modifies:
#   None
# Calls:
#   &recover_error
# Description:
#   If this is a restart, check the the value of the $state string against
#   what was read in from the 'recover_state' file on start up. The state
#   values are either true or false. True means the state string was logged 
#   out during the previous runs, false means it was not. &recover_open_state
#   set the values in the associative array %recover_state based on the
#   stings written out during previous runs.
#   If $recover_restart is false just return false.
#   See the comment for function recover_st_log for more information.
#
sub recover_st_chk {

    local($state) = @_[0];  # state string
    local($ret_status);     # Status return at end of CLEANUP block

    &recover_set_cur_function("recover_st_check");
    $ret_status = 0;

CLEANUP_ST_CHK: { # Always return at end of this block.

    #
    # Check the argument.
    #
    if(! $state) {

	#
	# A null string. Just return false.
	#
	$ret_status = 0;
	last CLEANUP_ST_CHK;
    }

    #
    # I don't trust uninitialized associatives. Look for a key matching 
    # the state. If we find one return its value. If not the default 0
    # gets returned.
    #
    foreach $key (keys(%recover_current_state)) {
	if($state eq $key) {
	    $ret_status = $recover_current_state{$state};
	}
    }
  } # CLEANUP_ST_CHK Always return here
    
    &recover_restore_cur_function();
    return $ret_status;   

}

#
# recover_st_log
# Inputs:
#   $state
#     state string.
# Outouts:
#   None
# Returns:
#   None
# References:
#   None
# Modifies:
#   None
# Calls:
#   &recover_error
# Description:
#   Write out the $state string. The string is of the form state=value.
#   The script uses these state values to determine what has 
#   already been done during previous runs in an attempt to start where
#   the last run left off. The strings are written out to the 'recover_state'
#   file in the root of the vob storage directory. On startup, if the file
#   exists it is read and elements of the associative array %recover_state
#   are set to true for each string read. My paranoia sets all known 
#   sting elements in 'recover_state' to false before the file is read.
#   Once the file is read it is closed and reopened in append mode for 
#   subsequent writes of addition state strings for the current run.
#   Related functions - &recover_open_state, recover_state_close,
#   &recover_st_chk.
#
sub recover_st_log  {

    local($state) = @_[0];         # The state string
    local($ret_status);            # Status return at end of CLEANUP block

    &recover_set_cur_function("recover_st_log");
    $ret_status = $RTN_NO_STATUS_SET; 

CLEANUP_ST_LOG: { # Always return at end of this block.
    #
    # If the $recover_state_is_open isn't set do nothing. Should raise
    # an error because this shouldn't have gotten called yet but....
    #
    if(!$recover_state_is_open) {

	last CLEANUP_ST_LOG;
    }
    
    #
    # Check the input.
    #
    if(! $state) {

	last CLEANUP_ST_LOG;
    }
    
    &recover_set_state($state);

  } # CLEANUP_ST_LOG Always return here

  &recover_restore_cur_function();
  return $ret_status;   

}

# 
# recover_check_path()
# Inputs:
#   $path 
#     Path to be checked.
#   $mode
#     Read or write. 
# Outouts:
#   None
# Returns:
#   None
# References:
#   None
# Modifies:
#   None
# Calls:
#   &recover_error
# Description:
#   Check the validity of the given path according to the given criteria.
#   A null path is assumed to be an internal error. Non existence is
#   a failure. On failure, format an error string.
#
sub recover_check_path {


    local($path) = @_[0];     # The path
    local($mode) = @_[1];     # The permission the path should have
    local($fstr) = "";        # A format buffer
    local($ret_status);       # Status return at end of CLEANUP block

    &recover_set_cur_function("recover_check_path");
    $ret_status = $RTN_NO_STATUS_SET; 

CLEANUP_CHECK_PATH: { # Always return at end of this block.
    #
    # Internal failure if its null.
    #
    if($path eq "") {
	
        $ret_status = $RTN_PATH_NULL;
	last CLEANUP_CHECK_PATH;
    }

    #
    # Fail if it doesn't exist
    #
    if(! -d $path) {

	$ret_status = $RTN_PATH_NOT_EXIST;
	last CLEANUP_CHECK_PATH;
    }

    #
    # Fail if it doesn't have the correct permissions.
    #

    if($mode eq $RECOVER_IS_WRITABLE) {

	if(! -w $path) {

	    $ret_status = $RTN_PATH_UNWRITABLE;
	    last CLEANUP_CHECK_PATH;
	}
    }
    elsif($mode eq $RECOVER_IS_READABLE) {

	if(! -r $path) {

	    $ret_status = $RTN_PATH_UNREADABLE;
	    last CLEANUP_CHECK_PATH;

	}
    }
    elsif($mode eq $RECOVER_IS_EXECUTABLE) {

	if(! -x $path) {

	    $ret_status = $RTN_PATH_NO_EXECUTE;
	    last CLEANUP_CHECK_PATH;
	}
    }
    else {

	$ret_status = $RTN_BAD_MODE;
	last CLEANUP_CHECK_PATH;
    }

    $ret_status = $RTN_GOOD_PATH;

  } # CLEANUP_CHECK_PATH Always return here

  &recover_restore_cur_function();
  return $ret_status;   

}


#
# recover_check_vob_info()
# Inputs: 
#   None
# Outputs: 
#   None
# References: 
#   $recover_arg_vob_tag
#   $recover_rgy_storage      
#   $recover_rgy_replica_uuid
#   $recover_rgy_family_uuid
#   $recover_rgy_host
# Modifies:
#   None
# Calls:
#   &mdep_command()
#   &recover_error
# Description:
#   Called on restart when the vob info has previously been set. Check
#   to make sure nothing was done between invocations that may cause
#   problems.
#
sub recover_check_vob_info {

    local($tag_or_store) = @_[0];   # What the second argument is
    local($arg2) = @_[1];           # Tag or storage
    local(@cmdbuf);                 # Return buffer for mdep_command
    local($line);                   # Tmp line buffer used to parse cmdbuf.
    local($ret_status);             # Status return at end of CLEANUP block
    local($uuid_digits);            # Unpunctuated replica uuid
    local($rep_uuid);               # Replica uuid
    local($fam_uuid);               # Family uuid
    local($storage);                # Local storage path
    local($host);                   # Host storage resides on

    &recover_set_cur_function("recover_check_vob_info");
    $ret_status = $RTN_NO_STATUS_SET; 

CLEANUP_CHECK_VOB_INFO: { # Always return at end of this block.
    # 
    # Do the lsvob to get the information.
    #
    if($tag_or_store eq "tag") {

	# 
	# Arg2 is a tag. 
	# Save NLSPATH and set to null for English
	# 
	$NLSPATH = $ENV{NLSPATH}; 
	$ENV{NLSPATH} = "";
	@cmdbuf = &mdep_command("$MDEP_LSVOB -long $arg2");
	#
	# Now restore NLSPATH 
	#
	$ENV{NLSPATH} = $NLSPATH; # Reset NLSPATH
    }
    else {

	#
	# Its a storage directory
	# Save NLSPATH and set to null for English
	# 
	$NLSPATH = $ENV{NLSPATH}; 
	$ENV{NLSPATH} = "";
	@cmdbuf = &mdep_command("$MDEP_LSVOB -long -storage $arg2");
	#
	# Now restore NLSPATH 
	#
	$ENV{NLSPATH} = $NLSPATH; # Reset NLSPATH
    }

    if(@cmdbuf[0]  eq $MDEP_COMMAND_FAILURE) {
	#
	# Just set error. Let the caller print the error if this call 
        # is to set the globals.
	#
	$ret_status = $RTN_NOT_OK;
	last CLEANUP_CHECK_VOB_INFO;
    }
 
    # 
    # Get the information
    #
    foreach $line (@cmdbuf) {

        chop $line;
	
	if($line =~ /Vob replica uuid: (.*)/) {
	    
	    $rep_uuid = $1; next;
	}
	
	if($line =~ /Vob family uuid:  (.*)/) {
	    
	    $fam_uuid = $1; next;
	}
        if($line =~ /Vob server access path: (.*)/) {
 
            $local = $1; next;
        }
 
        if($line =~ /  Server host: (.*)/) {
 
            $host = $1; next;
        }
	
        if($line =~ /  Global path: (.*)/) {

            $global = $1;
        }
    }
    if((lc($recover_rgy_replica_uuid) ne lc($rep_uuid)) ||
       (lc($recover_rgy_family_uuid) ne lc($fam_uuid))) {
	
	   $fstr = 
"A replica currently exists with information that differs from that in the
saved state for the previous invocation. 
  Saved replica uuid: $recover_rgy_replica_uuid,
  Saved family uuid: $recover_rgy_family_uuid,
  Current replica uuid: $rep_uuid,
  Current family uuid: $fam_uuid,
  Current host: $host
  Current local storage: $storage\n";
       
       &recover_error($fstr);
       $ret_status = $RTN_VOB_INFO_INCONSISTENT;
       last CLEANUP_CHECK_VOB_INFO;
       }
	
   
    $ret_status = $RTN_OK;

  } # CLEANUP_CHECK_VOB_INFO Always return here

    &recover_restore_cur_function();
    return $ret_status;   

}

# recover_get_vob_info()
# Inputs:
#   None
# Outputs:
#   None
# References:
#   $recover_arg_vob_tag
# Modifies:
#   $recover_rgy_storage
#   $recover_rgy_replica_uuid
#   $recover_rgy_family_uuid
#   $recover_rgy_host
# Calls:
#   &mdep_command()
#   &recover_error
# Description:
#   Obtains information from the local host's registry for the vob tag
#   specified on the command line and contained in $recover_arg_vob_tag.
#
sub recover_get_vob_info {
 
    local(@cmdbuf);        # Buffer to contain command output returned by
                           # mdep_command()
    local($line);          # Tmp line buffer used to parse cmdbuf.
    local($ret_status);    # Status return at end of CLEANUP block
    local($uuid_digits);   # Unpucuated replica uuid
    local($localp);
    local($global);
    local($command);
 
    &recover_set_cur_function("recover_get_vob_info");
    $ret_status = 1;
CLEANUP_GET_VOB_INFO: { # Always return at end of this block.
    #
    # Do the lsvob to get the information.
    # Save NLSPATH and set to null for English
    #
    $NLSPATH = $ENV{NLSPATH}; 
    $ENV{NLSPATH} = "";
    $command = "$MDEP_CLEARTOOL lsvob -long  $recover_arg_vob_tag";
    #
    # Now restore NLSPATH 
    #
    $ENV{NLSPATH} = $NLSPATH; # Reset NLSPATH
    if(&mdep_command_s(*command,*cmdbuf)) {
	
        $ret_status = 1;
        last CLEANUP_GET_VOB_INFO;
    }

    #
    # Set each of the rgy globals.
    #
    foreach $line (@cmdbuf) {
        chop $line;

        if($line =~ /Vob replica uuid: (.*)/) {
            $recover_rgy_replica_uuid = $1; next;
        }

        if($line =~ /Vob family uuid:  (.*)/) {
 
            $recover_rgy_family_uuid = $1; next;
        }

        if($line =~ /Vob server access path: (.*)/) {
 
            $recover_rgy_localp = $1;
            if($CLEARCASE_LIGHT) {
               $recover_rgy_globalp = $1;
            } 
            next;
        }

        if($line =~ /  Server host: (.*)/) {
 
            $recover_rgy_host = $1; next;
        }


        if($line =~ /Global path: (.*)/) {

            $recover_rgy_globalp = $1;
        }
    }

    $command = "$MDEP_CLEARTOOL describe -long  vob:$recover_arg_vob_tag";
    if(&mdep_command_s(*command,*cmdbuf)) {
        foreach $line (@cmdbuf) {
           chop $line;

	    if($line =~ /master replica:/) {
                $recover_replicated = 1;
                last;
	    }
        }
    }
 

    if(($recover_rgy_replica_uuid eq "") ||
       ($recover_rgy_family_uuid eq "") ||
       ($recover_rgy_localp eq "") ||
       ($recover_rgy_globalp eq "") ||
       ($recover_rgy_host eq "")) {
 
        $ret_status = 1;
        last CLEANUP_GET_VOB_INFO;
    }

 


 
    $ret_status = 0;
 
  } # CLEANUP_GET_VOB_INFO Always return here

  &recover_restore_cur_function();
  return $ret_status;
 
}

#
# recover_confirm_scenario()
# Inputs:
#   None
# Outputs:
#   None
# Returns:
#   None
# References
#   $recover_arg_snap_storage
#   $recover_arg_dest_storage
#   $recovery_scenario
#   $recover_rgy_storage
#   $recover_host_name
#   $recover_rgy_host
# Modifies:
#   None
# Calls:
#   &recover_check_path(()
#   &mdep_internal_error_str()
#   &recover_error
#   &mdep_prompt()
#   &recover_cleanup()
# Descriptions:
#   Does some consistency checking current parameters for the chosen
#   method. If all looks OK it tells the user a bit about the method 
#   and prompts for confirmation as to whether or not to proceed.
#
sub recover_confirm_scenario {

    local($snap_str) = "";      # sprintf buffer for the snapshot information
    local($scenario_str) = "";  # sprintf buffer for the scenario information
    local($fstr);               # General format string
    local($path_check);         # recover_check_path return string
    local($fmtinternal);        # Internal error format string
    local($ret_status);         # Status return at end of CLEANUP block

    &recover_set_cur_function("recover_confirm_scenario");
    $ret_status = $RTN_NO_STATUS_SET; 

CLEANUP_CONFIRM_SCENARIO: { # Always return at end of this block.

    if($recover_arg_snap_storage) {
        if($recover_snapath_complete) {
            $snap = &mdep_dirname($recover_arg_snap_storage);
        }
        else {
            $snap = $recover_arg_snap_storage;
        }
	$snap_str = sprintf($RECOVER_P_SNAP_CONFIRM,$snap);
    }

    #
    # For each scenario there is string which, once the restore parameters
    # are formatted in, will completely describe the chosen scenario along
    # with the selected parameters. The string includes a confirmation 
    # prompt. Some consistency checking is also done at this point raising
    # errors, either internal or user driven, on failure.
    #
    if($recover_scenario == $RECOVER_SC_UP_OS_OH_W_TEMP) {

	$fstr = sprintf($RECOVER_P_ACTIVE_OS_OH_WTEMP_CONFIRM,
			$recover_arg_dest_storage,
			$recover_host_name,
			$recover_arg_temp_storage,
			$snap_str);
	$reply = &mdep_prompt($fstr,
			      $MDEP_YES_QUIT,$MDEP_YES,$MDEP_NO_HELP);
	if(lc($reply) eq $MDEP_QUIT) {
	    
	    $ret_status = $RTN_QUIT;
	    last CLEANUP_CONFIRM_SCENARIO;
	}
    }
    elsif($recover_scenario == $RECOVER_SC_UP_OS_OH_NO_TEMP) {

	$fstr = sprintf($RECOVER_P_ACTIVE_OS_OH_NOTEMP_CONFIRM,
			$recover_arg_dest_storage,
			$recover_host_name,
			$snap_str);
	$reply = &mdep_prompt($fstr,
			      $MDEP_YES_QUIT,$MDEP_YES,$MDEP_NO_HELP);

	if(lc($reply) eq $MDEP_QUIT) {
	    
	    $ret_status = $RTN_QUIT;
	    last CLEANUP_CONFIRM_SCENARIO;
	}
    }
    elsif($recover_scenario == $RECOVER_SC_UP_NS_OH) {

	$fstr = sprintf($RECOVER_P_ACTIVE_NS_OH_CONFIRM,
			$recover_arg_dest_storage,
			$recover_host_name,
			$snap_str);
	
	$reply = &mdep_prompt($fstr,
			      $MDEP_YES_QUIT,$MDEP_YES,$MDEP_NO_HELP);
	if(lc($reply) eq $MDEP_QUIT) {
	    
	    $ret_status = $RTN_QUIT;
	    last CLEANUP_CONFIRM_SCENARIO;
	}
	
    }
    elsif($recover_scenario == $RECOVER_SC_UP_NS_NH) {

	$fstr = sprintf($RECOVER_P_ACTIVE_NS_NH_CONFIRM,
			$recover_arg_dest_storage,
			$recover_host_name,
			$snap_str);
	$reply = &mdep_prompt($fstr,
			      $MDEP_YES_QUIT,$MDEP_YES,$MDEP_NO_HELP);
	if(lc($reply) eq $MDEP_QUIT) {
	    
	    $ret_status = $RTN_QUIT;
	    last CLEANUP_CONFIRM_SCENARIO;
	}
    }
    elsif($recover_scenario == $RECOVER_SC_DOWN) {


	$fstr = sprintf($RECOVER_P_INACTIVE_CONFIRM,
			$recover_arg_dest_storage,
			$recover_host_name,
			$snap_str);
	
	$reply = &mdep_prompt($fstr,
			      $MDEP_YES_QUIT,$MDEP_YES,$MDEP_NO_HELP);
	if(lc($reply) eq $MDEP_QUIT) {
	    
	    $ret_status = $RTN_QUIT;
	    last CLEANUP_CONFIRM_SCENARIO;
	}
    }

    $ret_status = $RTN_OK;

  } # CLEANUP_CONFIRM_SCENARIO Always return here

  &recover_restore_cur_function();
  return $ret_status;   

}

#
# recover_get_scenario
# Inputs:
#   None
# Outputs:
#   None
# Returns:
#   scenario
# References: 
# Modifies:
# Calls:
# Descriptions:
#   This is called if no arguments were given. The user is prompted for all required
#   information.
#
sub recover_get_scenario {
    
    local($reply) = "";           # General reply buffer
    local($retrieved_data);       # Path of data retrieved from backup media
    local($fstr);

    while(!$reply) {

	if($recover_registered) {
	
	    if($MDEP_MACHINE eq 'NT') {
		$fstr = sprintf($RECOVER_P_VOB_STORAGE_IS_REGISTERED,
				$recover_rgy_globalp);
		$reply = &mdep_prompt($fstr,
				      $MDEP_ANY_RESPONSE,
				      $recover_rgy_globalp,
				      $RECOVER_H_VOB_STORAGE_IS_REGISTERED);
	    }
	    else {
		$fstr = sprintf($RECOVER_P_VOB_STORAGE_IS_REGISTERED,
				$recover_rgy_localp);
		$reply = &mdep_prompt($fstr,
				      $MDEP_ANY_RESPONSE,
				      $recover_rgy_localp,
				      $RECOVER_H_VOB_STORAGE_IS_REGISTERED);
	    }

	    if(lc($reply) eq $MDEP_QUIT) {
		
		return -1;
	    }       
	}
	else {

	    $reply = &mdep_prompt($RECOVER_P_VOB_STORAGE_IS_NOT_REGISTERED,
				  $MDEP_ANY_RESPONSE,
				  $MDEP_NO_DEFAULT,
				  $RECOVER_H_VOB_STORAGE_IS_NOT_REGISTERED);
	    
	    if(lc($reply) eq $MDEP_QUIT) {
		
		return -1;
	    }       
	}
    }
    $recover_arg_dest_storage = $reply;
    $recover_arg_dest_storage =~ s/^\s*(.*?)\s*$/$1/;
    $recover_state_dir = &mdep_dirname($recover_arg_dest_storage);

    $recover_registry_save_file = 
	$recover_state_dir.$MDEP_SEP.'register_save_'.&mdep_basename($recover_arg_vob_tag);
    &recover_set_state("recover_registry_save_file=$recover_registry_save_file");
    if($recover_registered) {
        if(&mdep_is_uncname($recover_arg_dest_storage)) {
            $recover_rgy_storage = $recover_rgy_globalp;
        }
        else {
            $recover_rgy_storage = $recover_rgy_localp;
        }
    }

    if(-e $recover_state_dir.$MDEP_SEP.$recover_state_file) {
        local($stat);
	$fstr = sprintf($RECOVER_P_IS_RESTART, $recover_state_dir);
	$reply = &mdep_prompt($fstr,
			      $MDEP_YES_QUIT,
			      $MDEP_YES,
			      $MDEP_NO_HELP);

	if(lc($reply) eq $MDEP_QUIT) {

	    return -1;
	}
	&recover_clear_inputs();
	if(!&recover_open_state_file()) {
	    &recover_error("Could not open previous state file.\n");
	    return -1;
	}
        &recover_set_inputs();
        $recover_restart = 1;
	$recover_do_save_state = 1;
        return $recover_scenario;
	
    }

    $reply = "";
    while(!$reply) {

	$reply = &mdep_prompt($RECOVER_P_RESTORED_DATA,
			      $MDEP_ANY_RESPONSE,
			      $MDEP_NO_DEFAULT,
			      $RECOVER_H_RESTORED_DATA);
    
	if(lc($reply) eq $MDEP_QUIT) {
	    
	    return -1;
	}
    }

    $retrieved_data = $reply;
    $retrieved_data =~ s/^\s*(.*?)\s*$/$1/;
    $recover_arg_back_storage = $retrieved_data;

    $reply = &mdep_prompt($RECOVER_P_SNAPSHOT,
			  $MDEP_ANY_RESPONSE,
			  $MDEP_NO_DEFAULT,
			  $RECOVER_H_SNAPSHOT);

    if(lc($reply) eq $MDEP_QUIT) {

	return -1;
    }

    if($reply) {
        $reply =~ s/^\s*(.*?)\s*$/$1/; 
	$recover_arg_snap_storage = $reply;
       
    }
    if(!$recover_registered &&
	-d $retrieved_data) {

	$fstr = sprintf($RECOVER_P_IS_RESTORE_IN_PLACE,
			$retrieved_data);

	$reply = &mdep_prompt($fstr,
			      $MDEP_YES_OR_NO_HELP,
			      $MDEP_NO,
			      $RECOVER_H_IS_RESTORE_IN_PLACE_UNREG);

	if(lc($reply) eq $MDEP_YES) {
	    
	    $reply = &mdep_prompt($RECOVER_P_WARN_1_REPLICATED_UNREG,
				  $MDEP_YES_OR_NO,
				  $MDEP_NO,
				  $MDEP_NO_HELP);
	    
	    if(lc($reply) eq $MDEP_YES) {
		if($recover_restart) {
		    return 0;
		}
		else {
		    return -1;
		}
	    }
	    &recover_set_state("recover_not_reg_data_restored");
	}
    }
	
    if($recover_registered &&
       &mdep_path_equal($recover_rgy_storage,$retrieved_data) &&
       mdep_same_hosts($recover_rgy_host,$recover_host_name) &&
       !&recover_st_chk("recover_st_new_registration") &&
       -d $retrieved_data) {

	&recover_set_state("recover_backup_equals_registered");
        $fstr = sprintf($RECOVER_P_IS_RESTORE_IN_PLACE,$retrieved_data);
	$reply = &mdep_prompt($fstr,
			      $MDEP_YES_OR_NO_HELP,
			      $MDEP_NO,
			      $RECOVER_H_IS_RESTORE_IN_PLACE);

	if(lc($reply) eq $MDEP_YES) {

	    if($recover_replicated) {

		$reply = &mdep_prompt($RECOVER_P_WARN_1_REPLICATED,
				      $MDEP_YES_OR_NO,
				      $MDEP_NO,
				      $MDEP_NO_HELP);
	    }
	    else {
		$reply = &mdep_prompt($RECOVER_P_WARN_1_NOT_REPLICATED,
				      $MDEP_YES_OR_NO,
				      $MDEP_NO,
				      $MDEP_NO_HELP);
	    }
	    if(lc($reply) eq $MDEP_YES) {
		if(recover_restart) {
		    return 0;
		}
		else {
		    return -1;
		}
	    }
            &recover_set_state("recover_backup_overwrite_done");
	    &recover_set_state("recover_st_old_storage_moved");
	}
    }
    if(!mdep_path_equal($retrieved_data,$recover_arg_dest_storage)) {

	    $recover_arg_temp_storage = $retrieved_data;
    }

    &recover_set_state("recover_arg_dest_storage=$recover_arg_dest_storage");

    if($recover_arg_temp_storage) {
	&recover_set_state(
	   "recover_arg_temp_storage=$recover_arg_temp_storage");
    }
    if($recover_arg_snap_storage) {
	&recover_set_state(
           "recover_arg_snap_storage=$recover_arg_snap_storage");
    }

   $scenario = &recover_determine_scenario();

    return $scenario;
}
    

#
# recover_set_scenario()
# Inputs: 
#   None
# Outputs: 
#   None
# Returns:
#   scenario
# References: 
#   $recover_rgy_storage
#   $recover_rgy_host
#   $recover_host_name
# Modifies:
#   $recover_scenario
#   $recover_arg_dest_storage
#   $recover_temp_storage
# Calls:
#   &recover_error
#   &mdep_prompt()
# Description:
#   Determine the scenario based on the following:
#     The original vob is down and there is sufficient disk space 
#     to restore local storage to a temporary location.
#       The vob is inactive. It will end up in the storage specified 
#       destination on the specified host. 
#               scenario = RECOVER_SC_DOWN
#       The vob will end up in the old storage
#               scenario = RECOVER_SC_UP_OS_OH
#       The vob will end up in a new storage location.
#         The new location is on the old host
#               scenario = RECOVER_SC_UP_NS_OH
#         The new location is on a new host
#               scenario = RECOVER_SC_UP_NS_NH
#     Due to disk space limitations the vob must be restored in 
#     place.
#               scenario = RECOVER_SC_IN_PLACE
#     Fall through 
#               scenario = RECOVER_SC_IS_UNKNOWN
#      
#   Old/new storage and old/new host are determined by examining the
#   appropriate argument variables and rgy variables. The user was
#   queried for whether the restore should be in place or not and
#   whether the vob is currently being accessed read only and locked.
#
#   If a temporary recovery area is needed and one was not given prompt
#   the user. 
#   
#   Fatal error if a required temporary storage area either doesn't exits
#   or isn't writable.
#   Fatal error if the destination storage area either doesn't exist or
#   isn't writable.
#   Fatal error on fall through to RECOVER_SC_IS_UNKNOWN.
#
sub recover_set_scenario {

    local($reply) = "";    # General reply buffer;
    local($scenario);      # Local scenario variable
    local($ret_status);    # Status return at end of CLEANUP block

    &recover_set_cur_function("recover_set_scenario");
    $ret_status = $RTN_NO_STATUS_SET; 
    $scenario = $RECOVER_SC_IS_UNKNOWN;

CLEANUP_SET_SCENARIO: { # Always return at end of this block.

    $scenario = &recover_determine_scenario();

    #
    # There is no default scenario. Fatal on unknown.
    #
    if($scenario == $RECOVER_SC_IS_UNKNOWN) {
	
	&recover_error("Could not determine scenario\n");
	last CLEANUP_SET_SCENARIO;
    }

  } # CLEANUP_SET_SCENARIO Always return here
    
    &recover_restore_cur_function();
    return $scenario;
}


sub recover_determine_scenario {

    local($scenario);

    if(!$recover_registered) {
	
	$scenario = $RECOVER_SC_DOWN;
    }
    elsif((&mdep_path_equal($recover_rgy_storage,$recover_arg_dest_storage)) &&
	  mdep_same_hosts($recover_host_name,$recover_rgy_host)) {

	    if($recover_arg_temp_storage) {
		$scenario = $RECOVER_SC_UP_OS_OH_W_TEMP;
	    }
	    else {
		$scenario = $RECOVER_SC_UP_OS_OH_NO_TEMP;
	    }
    }
    elsif(
        (!&mdep_path_equal($recover_rgy_storage,$recover_arg_dest_storage)) &&
	  mdep_same_hosts($recover_host_name,$recover_rgy_host)) {
	$scenario = $RECOVER_SC_UP_NS_OH;
    }
    elsif(mdep_same_hosts($recover_host_name,$recover_rgy_host)) {

	$scenario = $RECOVER_SC_UP_NS_NH;
    }

    return $scenario;

}

#
# recover_check_replica_uuid_file
# Inputs:
#   $restore_dir
#     The root of the restored vob directory.
# Outputs:
#   None
# Returns
#   None
# References:
#   None
# Modifies:
#   None
# Calls:
#   &mdep_internal_error_str
#   &recover_error 
#   &recover_cleanup()
# Description
#   Checkout the replica_uuid file in the restore area. If its not there
#   consider it an error. Tell the user the restore area appears to be
#   incomplete and exit. 
#
sub recover_check_replica_uuid_file {

    local($restore_dir) = @_[0];    # The restore area
    local($reppath);                # The path of the replica_uuid file
    local($found) = 0;              # Search token
    local($fstr);                   # General format string
    local($fminternal);             # Internal error format string
    local($found) = 0;              # Search token
    local($line);                   # File line buffer
    local($ret_status);             # Status return at end of CLEANUP block

    &recover_set_cur_function("recover_check_replica_uuid_file");
    $ret_status = $RTN_NO_STATUS_SET; 

CLEANUP_CHECK_REPLICA_UUID_FILE: { # Always return at end of this block.
    $reppath = $restore_dir.$MDEP_SEP.'replica_uuid';

    if((! -f $reppath) ||
       (! -r $reppath)) {

	#
	# Its not there, raise an error and cleanup. 
	#
	$ret_status = 0;
	last CLEANUP_CHECK_REPLICA_UUID_FILE;
    }

    #
    # Open the file 
    #
    unless(open(REPLICA_UUID,$reppath)) {
    
	#
	# This shouldn't have happened. Not sure what to do but treat it 
        # as an internal error for now.
	#
	$ret_status = 0;
	last CLEANUP_CHECK_REPLICA_UUID_FILE;
    }
    
    #
    # See if the file contains the same uuid that's in the registry. We 
    # don't document the format of this file so the format of the uuid stored 
    # in it may change. But the number of digits won't so remove all non 
    # alphanumerics and compare whats left. 
    #
    foreach $line (<REPLICA_UUID>) {
       
	local($uuid_digits);        # Oid file uuid - digits only
	local($rgy_uuid_digits);    # Registered family uuid - digits only

	chop $line;
	($uuid_digits = $line) =~ s/\W//g;
	if($uuid_digits &&
	   (length($uuid_digits) == 32)) {

	    $found = 1;

	    # 
	    # We found something that should be a uuid. If 
	    # $recover_registered is set, check it against the one
	    # in the registry. We'll save it in $recover_replica_uuid
            # in any case below so we'll know the replica uuid for this vob 
            # even if its not currently registered.
	    #
	    if($recover_registered) {

		($rgy_uuid_digits = $recover_rgy_replica_uuid) =~ s/\W//g;
		if(lc($uuid_digits) eq lc($rgy_uuid_digits)) {
	
		    #
		    # Its good.
		    #
		    $recover_replica_uuid = $uuid_digits;
		    $ret_status = 1;
		}
		else {

		    #
		    # Its not the right one. 
		    #
		    $ret_status = 0;
		}
	    }
	    else {

		$recover_replica_uuid = $uuid_digits;
		$ret_status = 1;
	    }

	    last;
	}
    }

    close(REPLICA_UUID);

    if(!$found) {

	#
	# Bad replica_uuid file
	#
	$ret_status = 0;
    }
    
  } # CLEANUP_CHECK_REPLICA_UUID_FILE Always return here

  &recover_restore_cur_function();
  return $ret_status;   

}

#
# recover_check_vob_oid_file
# Inputs:
#   $restore_dir
#     The root of the restored vob directory.
# Outputs:
#   None
# Returns
#   None
# References:
#   None
# Modifies:
#   None
# Calls:
#   &mdep_internal_error_str
#   &recover_error 
#   &recover_cleanup()
# Description
#   Checkout the vob_oid file in the restore area. If its not there
#   consider it an error. Tell the user the restore area appears to be
#   incomplete and exit. 
#
sub recover_check_vob_oid_file {

    local($restore_dir) = @_[0];    # The restore area
    local($oidpath);                # The path of the replica_uuid file
    local($found) = 0;              # Search token
    local($fstr);                   # General format string
    local($fminternal);             # Internal error format string
    local($found) = 0;              # Search token
    local($line);                   # File line buffer
    local($ret_status);             # Status return at end of CLEANUP block

    &recover_set_cur_function("recover_check_vob_oid_file");
    $ret_status = $RTN_NO_STATUS_SET; 

CLEANUP_CHECK_VOB_OID_FILE: { # Always return at end of this block.
    $oidpath = $restore_dir.$MDEP_SEP.'vob_oid';

    # 
    # First make sure the s directory is present. If not tell the
    # user to get it and wait. The user may choose to quit and that's
    # ok. If the user didn't quit go back and reread the top level
    # directory again and see if we were lied to. 
    #
    if((! -f $oidpath) ||
       (! -r $oidpath)) {

	#
	# Its not there, raise an error and cleanup. 
	#
	$ret_status = 0;
	last CLEANUP_CHECK_VOB_OID_FILE;
    }

    #
    # Open the file 
    #
    unless(open(VOB_OID,$oidpath)) {
    
	#
	# This shouldn't have happened. Not sure what to do but treat it 
        # as an internal error for now.
	#
	$ret_status = 0;
	last CLEANUP_CHECK_VOB_OID_FILE;
    }

    #
    # See if the file contains the same uuid that's in the registry. We 
    # don't document the format of this file so the format of the uuid stored 
    # in it may change. But the number of digits won't so remove all non 
    # alphanumerics and compare whats left. 
    #
    foreach $line (<VOB_OID>) {
	local($uuid_digits);        # Oid file uuid - digits only
	local($rgy_uuid_digits);    # Registered family uuid - digits only

	chop $line;
	($uuid_digits = $line) =~ s/\W//g;
	if($uuid_digits &&
	   (length($uuid_digits) == 32)) {

	    $found = 1;

	    # 
	    # We found something that should be a uuid. If 
	    # $recover_registered is set, check it against the one
	    # in the registry. We'll save it in $recover_replica_uuid
            # in any case below so we'll know the replica uuid for this vob 
            # even if its not currently registered.
	    #
	    if($recover_registered) {

		($rgy_uuid_digits = $recover_rgy_family_uuid) =~ s/\W//g;
		if(lc($uuid_digits) eq lc($rgy_uuid_digits)) {
	
		    #
		    # Its good.
		    #
		    $recover_family_uuid = $uuid_digits;
		    $ret_status = 1;
		}
		else {

		    #
		    # Its not the right one. 
		    #
		    $ret_status = 0;
		}
	    }
	    else {

		$recover_family_uuid = $uuid_digits;
		$ret_status = 1;
	    }

	    last;
	}
    }

    close(VOB_OID);

    if(!found) {

	#
	# Bad vob_oid file.
	#
	$ret_status = 0;
    }

  } # CLEANUP_CHECK_REPLICA_UUID_FILE Always return here

  &recover_restore_cur_function();
  return $ret_status;   

}

#
# recover_check_snapshot_area
# Inputs: 
#   None
# Outputs:
#   None
# Returns
#   None
# References:
# Modifies:
# Calls:
# Description:
#   Make sure the snapshot area that was given is the right one for this
#   vob. From the snap_sata file in the snapshot area check the replica
#   uuid and the vob oid against the corresponding values from the registry.
#   Check that the snapshot was successful by looking at the status stored
#   in the same file. Also check that a dbcheck was done. If not ask the
#   user if one should be done at this time. A check is done on each 
#   snapped file by comparing the size snapped to the current size. The
#   snapped size is stored in the snap_data file.
#   
sub recover_check_snapshot_area {

    local($sdpath);                   # Path for the snap_data file
    local($line);                     # Line buffer for reading files
    local($ssize);                    # A file's snap size
    local($csize);                    # The file's current size
    local($snp_repuuid);              # Snapped replica uuid
    local($snp_voboid);               # Snapped family uuid
    local($snp_status);               # Snapped status
    local($snp_dbcheck);              # Snapped dbcheck status
    local($file);                     # Snapped file paths
    local($directory);                # Snapped directory path
    local($ret_status);               # Status return at end of CLEANUP block


    &recover_set_cur_function("recover_check_snapshot_area");
    $ret_status = $RTN_NO_STATUS_SET; 

CLEANUP_CHECK_SNAPSHOT_AREA: { # Always return at end of this block.

    
    $sdpath = $recover_arg_snap_storage.$MDEP_SEP.'snap_data';

    unless(open(SNAP_DATA_FILE,"$sdpath")) {
	#
	# Probably a bad snap. Tell the user the snap area appears to
        # be incomplete and call cleanup.
	#
	$fstr = sprintf($RECOVER_E_NO_SNAP_DATA,$sdpath);
	&recover_error($fstr);
	$ret_status = $RTN_NOT_OK;
	last CLEANUP_CHECK_SNAPSHOT_AREA;
    }
    
    #
    # Parse the snap_data file for the information required for the
    # checks. Check the current sizes against the size in the snap_data
    # file as we come across them in the later.
    #    
    foreach $line (<SNAP_DATA_FILE>) {

	if($line =~ /ReplicaUUID:(.*)/) {

	    $snp_repuuid = $1; next;
	} 
	if($line =~ /FamilyUUID:(.*)/) {
	    $snp_voboid = $1; next;
	} 
	if($line =~ /SnapStatus:(.*)/) {
	    $snp_status = $1; next;
	} 
	if($line =~ /Dbcheck:(.*)/) {
	    $snp_dbcheck = $1; next;
	}
	if($line =~ /File:(.*) (.*)/) {

	    $file = $recover_arg_snap_storage.$MDEP_SEP.$1;
	    $ssize = $2;
	    $csize = -s $file;
	    if($ssize != $csize) {

		#
		# If a file differs, raise an error and cleanup. 
		#
		&recover_error("Snapped file has bad size \"$file\"\n");
		$ret_status = $RTN_NOT_OK;
		last CLEANUP_CHECK_SNAPSHOT_AREA;
	    }
	}
    }

    # 
    # Check the snapped status first. If its bad no use going on.
    #
    if($snp_status ne "Success") {

       if($snp_status eq "Questionable") {
 
            if($snp_dbcheck eq "Succeeded") {
 
                #
                # The snap was done without the lock held but a dbcheck
                # was done and it was clean. Upgrade the status to good.
                #
                $snp_status = "Success";
            }
        }
        else {
 
            &recover_error("The snap has a bad status\n");
            $ret_status = $RTN_NOT_OK;
            last CLEANUP_CHECK_SNAPSHOT_AREA;
        }
    }

    #
    # Check the replica uuid and vob oid. Don't rely on the format being 
    # the same. Prune all but hex digits before doing the compare for both.
    #
    ($snp_repuuid_digits = $snp_repuuid) =~ s/\W//g;
    ($rgy_repuuid_digits = $recover_replica_uuid) =~ s/\W//g;
    if(lc($snp_repuuid_digits) ne lc($rgy_repuuid_digits)) {

	&recover_error("The snapped data has the wrong replica uuid\n");
	$ret_status = $RTN_NOT_OK;
	last CLEANUP_CHECK_SNAPSHOT_AREA;
    }

    ($snp_voboid_digits = $snp_voboid) =~ s/\W//g;
    ($rgy_voboid_digits = $recover_family_uuid) =~ s/\W//g;
    if(lc($snp_voboid_digits) ne lc($rgy_voboid_digits)) {

	&recover_error("The snapped data has the wrong family oid\n");
	$ret_status = $RTN_NOT_OK;
	last CLEANUP_CHECK_SNAPSHOT_AREA;
    }
NOT_IF_DBCHECK_DONE: {

    if(&recover_st_chk("recover_st_dbcheck_done")) {

	#
	# We're on a restart and we already determined that the data
        # base either wasn't required or has been done.
	#
	last NOT_IF_DBCHECK_DONE;
    }

    #
    # Now look at the dbcheck status. If it was marked as Succeeded then
    # the database is known to be clean. If its marked Failed then it was 
    # done at the time of the snap and errors were found. Raise the error
    # and cleanup. If it just wasn't done then offer the user the chance
    # to get it done now. 
    #
    if($snp_dbcheck eq "Succeeded") {
        
	&recover_set_state("recover_st_dbcheck_done");
    }
    elsif($snp_dbcheck eq "Failed") {
	
	    #
	    # It failed during the snap. Cleanup and go away.
	    #
	    &recover_error("The snapped data failed dbcheck\n");
	    &recover_unset_state("recover_st_dbcheck_done");
	    $ret_status = $RTN_NOT_OK;
	    last CLEANUP_CHECK_SNAPSHOT_AREA;
	}
    else {

	#
	# Wasn't done apparently. Ask if the user wanst to do it now.
        # Later when the replica is brought up, if it is determined that the
	# backup was done without the lock and the user answers no here,
        # a dbcheck will be forced.
        # 
	$reply = &mdep_prompt($RECOVER_P_NEEDS_DBCHECK,
				   $MDEP_YES_NO_OR_QUIT,
				   $MDEP_NO,
				   $RECOVER_H_NEEDS_DBCHECK);
	if(lc($reply) eq $MDEP_QUIT) {

	    &recover_unset_state("recover_st_dbcheck_done");
	    $ret_status = $RTN_NOT_OK;
	    last CLEANUP_CHECK_SNAPSHOT_AREA;
	}
	elsif(lc($reply) eq $MDEP_YES) {
	    print "\nPerforming database check. This may take some time...";
	    $status = &mdep_dodbcheck($recover_arg_snap_storage);
	    if($status ne $RTN_DBCHECK_OK) {
	        
		&recover_error("Dbcheck failed\n");
		&recover_unset_state("recover_st_dbcheck_done");
		$ret_status = $RTN_NOT_OK;
		last CLEANUP_CHECK_SNAPSHOT_AREA;
	    }
	    &recover_set_state("recover_st_dbcheck_done");
            print "checked clean \n";
	}
    }

  } # NOT_IF_DBCHECK_CLEAN

    $ret_status = $RTN_OK;

  } # CLEANUP_CHECK_SNAPSHOT_AREA Always return here

  &recover_restore_cur_function();
  return $ret_status;   

}

#
# recover_check_restore
# Inputs:
#   $restore_dir 
#     The location of the restored date
# Outputs:
#   None
# Returns:
#   None
# References:
#
# Modifies:
#
# Calls:
#
# Description:
#   Check the restored root data. Prompt the user for verification that
#   all of the pools have been restored properly indicating which ones 
#   may be remote - the pool is a symlink on unix. Verify that the replica
#   uuid and family oid files contain the same indentities as the registry
#   contained for this vob. If a snapshot directory was given a call to
#   recover_snapshot is made. If not the recovered database is sanity 
#   checked.
#

sub recover_check_restore {

    local($restore_dir) = @_[0];    # The restored directory
    local($fmtinternal);            # Format on internal error
    local($fstr);                   # General format string
    local($entry);                  # An entry in the vob root
    local(@entries);                # Entries in the vob's root directory
    local($dbpath);                 # Path for the restored database.
    local($required);               # A required entry
    local($reply);                  # Reply buffer
    local($path_check);             # recover_check_path return string
    local($status);                 # General status
    local($ret_status);             # Status return at end of CLEANUP block

    &recover_set_cur_function("recover_check_restore");
    $ret_status = $RTN_NO_STATUS_SET; 

CLEANUP_CHECK_RESTORE: { # Always return at end of this block.

    if((mdep_path_equal($restore_dir,$recover_arg_temp_storage)) &&
       &recover_st_chk("recover_temp_moved")) {

	$ret_status = $RTN_OK;
	last CLEANUP_CHECK_RESTORE;
    }

    while( ! -d $restore_dir) {
	$fstr = sprintf($RECOVER_P_GET_RESTORED_DATA,$restore_dir);
	$reply = &mdep_prompt($fstr,
			      $MDEP_QUIT_RETURN_TO_CONTINUE,
			      $MDEP_NO_DEFAULT,
			      $MDEP_NO_HELP);

	if(lc($reply) eq $MDEP_QUIT) {
	    
	    $ret_status = $RTN_NOT_OK;
	    last CLEANUP_CHECK_RESTORE;
	}
        &recover_set_state("recover_backup_overwrite_done");
    }

    if(&recover_st_chk("recover_backup_equals_registered") &&
	!&recover_st_chk("recover_backup_overwrite_done")) {

        $fstr = sprintf($RECOVER_P_GET_RESTORED_DATA,$restore_dir);
	$reply = &mdep_prompt($fstr,
		    $MDEP_QUIT_RETURN_TO_CONTINUE,
		    $MDEP_NONE,
		    $MDEP_NO_HELP);

	if(lc($reply) eq $MDEP_QUIT) {
	    
	    $ret_status = $RTN_NOT_OK;
	    last CLEANUP_CHECK_RESTORE;
	}
        &recover_set_state("recover_backup_overwrite_done");
    }

    if(!&recover_check_replica_uuid_file($restore_dir)) {

	&recover_error("The restored data has the wrong replica uuid\n");
	$ret_status = $RTN_NOT_OK;
	last CLEANUP_CHECK_RESTORE;
    }
    if(!&recover_check_vob_oid_file($restore_dir)) {

	&recover_error("The restored data has the wrong vob oid\n");
	$ret_status = $RTN_NOT_OK;
	last CLEANUP_CHECK_RESTORE;
    }

    if($recover_arg_snap_storage) {

	if(!$recover_snapath_complete) {

	    $recover_arg_snap_storage = 
		$recover_arg_snap_storage.$MDEP_SEP.$recover_replica_uuid;
	    $recover_snapath_complete = 1;
	}
	$status = &recover_check_snapshot_area();
	if($status ne $RTN_OK) {
            
	    $ret_status = $status;
	    last CLEANUP_CHECK_RESTORE;
	}
    }

    if(!&recover_st_chk("recover_verify_pools")) {
	$reply = &mdep_prompt($RECOVER_P_VERIFY_POOLS,
			      $MDEP_QUIT_RETURN_TO_CONTINUE,
			      $MDEP_NONE,
			      $RECOVER_H_VERIFY_POOLS);
	if(lc($reply) eq $MDEP_QUIT) {

	    $ret_status = $RTN_NOT_OK;
	    last CLEANUP_CHECK_RESTORE;
	}
        &recover_set_state("recover_verify_pools");
    }

    $dbpath = $restore_dir.$MDEP_SEP.'db';
    &mdep_rmfile($dbpath.$MDEP_SEP.'vista.taf');
    &mdep_rmfile($dbpath.$MDEP_SEP.'delete_list.db');
    
    $ret_status = $RTN_OK;

  } # CLEANUP_CHECK_RESTORE Always return here

  &recover_restore_cur_function();
  return $ret_status;   

}

#
# recover_protect_data_base
# Inputs:
#   $restore_dir 
#     The location of the restored date
# Outputs:
#   None
# Returns:
#   None
# References:
#
# Modifies:
#
# Calls:
#
# Description:
#   Move aside the schema file. Until its put back when its safe no reads
#   or writes of the database will work. The vob may still be registered
#  since the registry doesn't know about the database.
#
sub recover_protect_data_base {
    
    local($restore_dir) = @_[0];       # The restored directory
    local($dbpath);                    # The database path
    local($schemapath);                # Path of the schema file
    local($schemapath_recover);        # Renamed schema file
    local($ret_status);                # Status return at end of CLEANUP block

    &recover_set_cur_function("recover_protect_data_base");
    $ret_status = $RTN_NO_STATUS_SET; 

CLEANUP_PROTECT_DATA_BASE: { # Always return at end of this block.

    #
    # Check the input.
    #
    if(! $restore_dir) {

	$ret_status = $RTN_PATH_NULL;
	last CLEANUP_PROTECT_DATA_BASE;
    }
    $dbpath = $restore_dir.$MDEP_SEP.'db';
    $schemapath = $dbpath.$MDEP_SEP.'vob_db_schema_version';
    $schemapath_recover = $schemapath .'.recover';
	  
    # 
    # Check the database directory for write.
    # 
    if(! -w $dbpath) {

	$ret_status = $RTN_PDIR_UNWRITABLE;
	last CLEANUP_PROTECT_DATA_BASE;
    }
    #
    # Do the rename.
    #
    if(! rename($schemapath,$schemapath_recover)) {

	$ret_status = $RTN_RENAME_FAILED;
	last CLEANUP_PROTECT_DATA_BASE;
    }

    $ret_status = $RTN_DB_PROTECTED;

  } # CLEANUP_PROTECT_DATA_BASE Always return here

  &recover_restore_cur_function();
  return $ret_status;   

}



#
# recover_unprotect_data_base
# Inputs:
#   $restore_dir 
#     The location of the restored date
# Outputs:
#   None
# Returns:
#   None
# References:
#
# Modifies:
#
# Calls:
#
# Description:
#   Move the schema file back. The vob is open for business at this
#   point.
#
sub recover_unprotect_data_base {
    
    local($restore_dir) = @_[0];       # The restored directory
    local($dbpath);                    # The database path
    local($schemapath);                # Path of the schema file
    local($schemapath_recover);        # Renamed schema file
    local($ret_status);                # Status return at end of CLEANUP block
	  
	  
    &recover_set_cur_function("recover_unprotect_data_base");
    $ret_status = $RTN_NO_STATUS_SET; 
CLEANUP_UNPROTECT_DATA_BASE: { # Always return at end of this block.

    if(! $restore_dir) {

	$ret_status = $RTN_PATH_NULL;
	last CLEANUP_UNPROTECT_DATA_BASE;
    }
    $dbpath = $restore_dir.$MDEP_SEP.'db';
    $schemapath = $dbpath.$MDEP_SEP.'vob_db_schema_version';
    $schemapath_recover = $schemapath .'.recover';
	  
    # 
    # Check the database directory for write.
    # 
    if(! -w $dbpath) {

	$ret_status = $RTN_PDIR_UNWRITABLE;
	last CLEANUP_UNPROTECT_DATA_BASE;
    }
    #
    # Do the rename.
    #
    if(! rename($schemapath_recover,$schemapath)) {
	
	$ret_status = $RTN_RENAME_FAILED;
	last CLEANUP_UNPROTECT_DATA_BASE;
    }
    $ret_status = $RTN_DB_UNPROTECTED;

  } # CLEANUP_UNPROTECT_DATA_BASE Always return here

  &recover_restore_cur_function();
  return $ret_status;   

}

#
# recover_restore_snapshot
# Inputs:
#   $restore_dir 
#     The location of the restored date
# Outputs:
#   None
# Returns:
#   Good snap recover - $RTN_OK
#   Otherwise - $RTN_NOT_OK
# References:
#   @recover_snapped_directories
#   @recover_snapped_files
# Modifies:
#   None
# Calls:
#   &recover_error
#   &recover_cleanup
#   &mdep_copy_directory
#   &mdep_copy_file
# Description:
#   Restore the snapshot referenced by $recover_arg_snap_storage to the
#   restore storage area $restore_dir. All the integrity checks of the snapped
#   area have already been done. 
#
sub recover_restore_snapshot {

    local($restore_dir)= @_[0];    # The restored directory
    local($directory);             # Directories that must exist or be created
    local($file);                  # Files that must be copied 
    local($sdpath);                # Directory in the snapped area
    local($rdpath);                # Directory in the restore area
    local($stat_str);              # Copy function return strings
    local($fstr);                  # General format string
    local(@snapobjs);              # Objects at top level of snap area
    local($rpath);                 # Path for object in restore area
    local($spath);                 # Path for object in snap area
    local($dbpath);                # Path for the restored db directory
    local($reply);                 # Prompt reply
    local($fstr);                  # General format buffer
    local($ret_status);            # Status return at end of CLEANUP block

    
    &recover_set_cur_function("recover_restore_snapshot");
    $ret_status = $RTN_NO_STATUS_SET; 

CLEANUP_RESTORE_SNAPSHOT: { # Always return at end of this block.

    #
    # Open the snap area and read the top level. For directories
    # call copy_directory and for files call copy_files. Check the 
    # restore area for existence of anything about to be copied in
    # and prompt the user for verification to nuke the old restore
    # version before copying in the new.
    #
    unless(opendir(SNAP_DIR,$recover_arg_snap_storage)) {

	#
	# Treat this as an internal error. We should not have gotten
        # this far with a bad snap path string.
	#
	&recover_error("Could not open snap storage \"$recover_arg_snap_storage\": $!\n");
	$ret_status = $RTN_NOT_OK;
	last CLEANUP_RESTORE_SNAPSHOT;
    }
    
    @snapobjs = readdir(SNAP_DIR);
    close(SNAP_DIR);

    foreach $object (@snapobjs) {

	if(&mdep_skip_directory($object) == $RTN_TRUE) {

	    next;
	}
	if($object eq "snap_data") {

	    next;
	}

	#
	# Make the snap and restore paths for the object.
	#
	$opath = $recover_arg_snap_storage.$MDEP_SEP.$object;
        $rpath = $restore_dir.$MDEP_SEP.$object;

	if(-d $opath) {

	    #
	    # If an old copy of the directory is in the restore storage
            # area prompt the user for verification and nuke it.
            # Quit allowed
            #
	    if(-d $rpath) {

		$fstr = sprintf($RECOVER_P_REMOVE_OLD_RESTORE_DIR,$rpath);
		$reply = &mdep_prompt(
				   $fstr,
				   $MDEP_YES_HELP_QUIT,
				   $MDEP_YES,
				   $RECOVER_H_REMOVE_OLD_RESTORE_DIR);
		if(lc($reply) eq $MDEP_QUIT) {
		    
		    $ret_status = $RTN_NOT_OK;
		    last CLEANUP_RESTORE_SNAPSHOT;
		}
                if($MDEP_MACHINE eq 'NT') {
                    system_redirect("rmdir /S /Q $rpath");
                }
                else {
		    rmdir($rpath);
                }
	    }

	    $stat_str = &mdep_copy_directory($opath,$restore_dir);

	    if($stat_str ne $RTN_COPY_OK) {

		&recover_error("Directory copy, snap to restore, failed\n  \"$directory\" to \"$restore_dir\"\n");
		$ret_status = $RTN_NOT_OK;
		last CLEANUP_RESTORE_SNAPSHOT;
	    }
	}
	else {

	    #
	    # If an old copy of the file is in the restore storage
            # area prompt the user for verification and nuke it.
            # Quit allowed
            #
	    if(-f $rpath) {
		$fstr = sprintf($RECOVER_P_REMOVE_OLD_RESTORE_FILE,
				$rpath);
		$reply = &mdep_prompt(
				   $fstr,
				   $MDEP_YES_HELP_QUIT,
				   $MDEP_YES,
				   $RECOVER_H_REMOVE_OLD_RESTORE_FILE);
		if(lc($reply) eq $MDEP_QUIT) {

		    $ret_status = $RTN_NOT_OK;
		    last CLEANUP_RESTORE_SNAPSHOT;
		}
		&mdep_rmfile($rpath);
	    }

	    $stat_str = &mdep_copy_file($opath,$restore_dir);

	    if($stat_str ne $RTN_COPY_OK) {
		
		&recover_error("File copy, snap to restore, failed\n  \"$directory\" to \"$restore_dir\"\n");
		$ret_status = $RTN_NOT_OK;
		last CLEANUP_RESTORE_SNAPSHOT;
	    }
	}
    }
    
    #
    # Be paranoid about the transaction log. If the snap was done without 
    # the lock we don't want anything still in the journal to get rolled 
    # into the database at this point. Remove all three vista files. They
    # will get regenerated when the vob starts up.
    #
    $dbpath = $restore_dir.$MDEP_SEP.'db';
    &mdep_rmfile($dbpath.$MDEP_SEP.'vista.taf');
    &mdep_rmfile($dbpath.$MDEP_SEP.'delete_list.db');

    $ret_status = $RTN_OK;
   
    
    
  } # CLEANUP_RESTORE_SNAPSHOT Always return here

  &recover_restore_cur_function();
  return $ret_status;   

}	    

#
# recover_copy_temp_to_dest
# Inputs:
#   None
# Outputs:
#   None
# Returns:
#   Success $RTN_OK
#   Otherwise $RTN_NOT_OK
# References:
#   $recover_arg_dest_storage
#   $recover_arg_temp_storage
# Modifies:
#   None
# Calls:
#   &recover_error
#   &mdep_skip_directory
#   &mdep_copy_file
# Description:
#   Copy the temporary storage data to its final destination.
#
sub recover_copy_temp_to_dest {

    local($fstr);             # General format string
    local(@objects);          # File systemp objects
    local($object);           # One of them
    local($object_path);      # Full path for $object
    local($status);           # General status
    local($ret_status);       # Return status

    &recover_set_cur_function("recover_copy_temp_to_dest");
    $ret_status = $RTN_NO_STATUS_SET; 

   
CLEANUP_COPY_TEMP_TO_DEST: { # Always return at end of this block.
    #
    # Make the destination directory.
    #

    while(! -d $recover_arg_temp_storage) {

        $fstr = sprintf($RECOVER_P_GET_RESTORED_DATA,$recover_arg_temp_storage);
	$reply = &mdep_prompt($fstr,
		    $MDEP_QUIT_RETURN_TO_CONTINUE,
		    $MDEP_NONE,
		    $MDEP_NO_HELP);

	if(lc($reply) eq $MDEP_QUIT) {
	    
	    $ret_status = $RTN_NOT_OK;
	    last CLEANUP_CHECK_RESTORE;
	}
    }

    if(! mkdir($recover_arg_dest_storage,0755)) {

	&recover_error("Failed to make temporary store directory\n  \"$recover_arg_temp_storage\": $!\n");
        $ret_status = 0;
	last CLEANUP_COPY_TEMP_TO_DEST;
    }
	
    
    #
    # Now read the temp directory contents.
    #
    unless(opendir(DESTINATION_COPY,$recover_arg_temp_storage)) {
	    
	&recover_error("Could not open temporary store \"$recover_arg_temp_storage\": $!\n");
	$ret_status = 0;
	last CLEANUP_COPY_TEMP_TO_DEST;
    }
    
    @objects = readdir(DESTINATION_COPY);
    close(DESTINATION_COPY);
    
	
    # 
    # Copy each object. 
    #
    foreach $object (@objects) {

	
	if(&mdep_skip_directory($object)) {

	    #
	    # skip the those that should not be copied.
	    #
	    next;
	}
	
	$object_path = $recover_arg_temp_storage.$MDEP_SEP.$object;
	
	if(-d $object_path) {

 	    $status = &mdep_copy_directory($object_path,
					   $recover_arg_dest_storage);

	    if($status ne $RTN_COPY_OK) {
		    
		    &recover_error("Directory copy temporary to destination directory failed\n  \"$object_path\" to \"$recover_arg_dest_storage\"\n");
		    $ret_status = 0;
		    last CLEANUP_COPY_TEMP_TO_DEST;
	    }
	}
	else {

	    $status = &mdep_copy_file($object_path,
				      $recover_arg_dest_storage);
	    if($status ne $RTN_COPY_OK) {
		
		&recover_error("File copy temporary to destination directory failed\n  \"$object_path\" to \"$recover_arg_dest_storage\"\n");
		$ret_status = 0;
		last CLEANUP_COPY_TEMP_TO_DEST;
	    }
	}
    }

    $ret_status = 1;

  } # CLEANUP_COPY_TEMP_TO_DEST Always return here

    &recover_restore_cur_function();
    return $ret_status;
}


sub recover_restore_storage {

    local($fstr);               # General format buffer
    local($reply);              # Reply buffer
    local($status);             # General status
    local($ret_status);         # Return status

    &recover_set_cur_function("recover_restore_storage");
    $ret_status = $RTN_NO_STATUS_SET; 

CLEANUP_RESTORE_STORAGE: { # Always return at end of this block.

    if($recover_scenario == $RECOVER_SC_DOWN) {

	if(!&recover_st_chk("recover_not_reg_data_restored")) {

	    if(-d $recover_arg_back_storage) {

		
                $fstr = sprintf($RECOVER_P_GET_RESTORED_DATA,$recover_arg_back_storage);
		$reply = &mdep_prompt($fstr,
				      $MDEP_QUIT_RETURN_TO_CONTINUE,
				      $MDEP_NONE,
				      $MDEP_NO_HELP);
		
		if(lc($reply) eq $MDEP_QUIT) {
		    $ret_status = $RTN_NOT_OK;
		    last CLEANUP_RESTORE_STORAGE;
		}
	    }
	    else {

		while(! -d $recover_arg_back_storage) {
		    
		    $fstr = sprintf($RECOVER_P_GET_RESTORED_DATA,$recover_arg_back_storage);
		    $reply = &mdep_prompt($fstr,
					  $MDEP_QUIT_RETURN_TO_CONTINUE,
					  $MDEP_NONE,
					  $MDEP_NO_HELP);
		    
		    if(lc($reply) eq $MDEP_QUIT) {
			$ret_status = $RTN_NOT_OK;
			last CLEANUP_RESTORE_STORAGE;
		    }
		}
	    }
	}

	&recover_set_state("recover_not_reg_data_restored");
	#
	# Check it out
	#
	$status = &recover_check_restore($recover_arg_back_storage);
	if($status ne $RTN_OK) {
	    
	    $ret_status = $RTN_NOT_OK;
	    last CLEANUP_RESTORE_STORAGE;
	}

	#
	# If a snapshot area was given merge it into the restore area. 
	#
	if($recover_arg_snap_storage  &&
	   (!&recover_st_chk("recover_st_snapshot_restored"))) {
	    
	    
	    $status = &recover_restore_snapshot($recover_arg_back_storage);
	    if($status ne $RTN_OK) {
		&recover_cleanup();
	    }
	    &recover_set_state("recover_st_snapshot_restored");
	}
	
	if($recover_arg_temp_storage) {

	    $fstr = sprintf($RECOVER_P_MOVE_TEMP_TO_DEST,
			    $recover_arg_temp_storage,
			    $recover_arg_dest_storage);
	    
	    $reply = &mdep_prompt($fstr,
				  $MDEP_YES_HELP_QUIT,
				  $MDEP_YES,
				  $RECOVER_H_MOVE_TEMP_TO_DEST);
	    if(lc($reply) eq $MDEP_QUIT) {
		
		$ret_status = $RTN_NOT_OK;
		last CLEANUP_RESTORE_STORAGE;
	    }
	    
	    while(-e $recover_arg_dest_storage) {
		
		$fstr = sprintf($RECOVER_P_STORE_ALREADY_EXISTS,
				$recover_arg_dest_storage);
		
		$reply = &mdep_prompt($fstr,
			     $MDEP_QUIT_RETURN_TO_CONTINUE,
			     $MDEP_NONE,
			     $MDEP_NO_HELP);
		
		if(lc($reply) eq $MDEP_QUIT) {
		    
		    $ret_status = $RTN_NOT_OK;
		    last CLEANUP_RESTORE_STORAGE;
		}
	    }


	    if(!&recover_st_chk("recover_temp_moved")) {
		if(&mdep_move_temp_to_dest($recover_arg_temp_storage,
					   $recover_arg_dest_storage)) {
		    
		    if(!&recover_copy_temp_to_dest()) {
			
			$ret_status = $RTN_NOT_OK;
			last CLEANUP_RESTORE_STORAGE;
		    }
		}
		&recover_set_state("recover_temp_moved");
	    }
	}
    }
    elsif($recover_scenario == $RECOVER_SC_UP_OS_OH_NO_TEMP) {

	#
	# Check out the restored data.
	#
	$status = &recover_check_restore($recover_arg_back_storage);
	if($status ne $RTN_OK) {
	    
	    &recover_cleanup();
	}

	#
	# Ifd a snapshot area was given merge it into the restore area. Unless
	# the restoration is in place - it will be done later.
	#
	if($recover_arg_snap_storage  &&
	   (!&recover_st_chk("recover_st_snapshot_restored"))) {
	    
	    
	    $status = &recover_restore_snapshot($recover_arg_back_storage);
	    if($status ne $RTN_OK) {
		&recover_cleanup();
	    }
	    &recover_set_state("recover_st_snapshot_restored");
	}
    }
    elsif($recover_arg_temp_storage) {

 	$fstr = sprintf($RECOVER_P_MOVE_TEMP_TO_DEST,
			$recover_arg_temp_storage,
			$recover_arg_dest_storage);
	
	$reply = &mdep_prompt($fstr,
			      $MDEP_YES_HELP_QUIT,
			      $MDEP_YES,
			      $RECOVER_H_MOVE_TEMP_TO_DEST);
	if(lc($reply) eq $MDEP_QUIT) {
	    
	    $ret_status = $RTN_NOT_OK;
	    last CLEANUP_RESTORE_STORAGE;
	}

	while(-e $recover_arg_dest_storage) {

            $fstr = sprintf($RECOVER_P_STORE_ALREADY_EXISTS,
			    $recover_arg_dest_storage);

	    $reply = &mdep_prompt($fstr,
			 $MDEP_YES_QUIT,
			 $MDEP_YES,
			 $MDEP_NO_HELP);

	    if(lc($reply) eq $MDEP_QUIT) {

		$ret_status = $RTN_NOT_OK;
		last CLEANUP_RESTORE_STORAGE;
	    }
	}

	if(!&recover_st_chk("recover_temp_moved")) {
	    if(&mdep_move_temp_to_dest($recover_arg_temp_storage,
				       $recover_arg_dest_storage)) {
		
		if(!&recover_copy_temp_to_dest()) {
		    
		    $ret_status = $RTN_NOT_OK;
		    last CLEANUP_RESTORE_STORAGE;
		}
	    }
	    &recover_set_state("recover_temp_moved");
	}
    }
   
    #
    # Move the database schema file aside temporarily so no access
    # to the database will be allowed until we are good and ready.
    # If the snapshot is being done in place the database isn't there
    # yet.
    #
    if(!&recover_st_chk("recover_st_protected")) {


	$status = &recover_protect_data_base($recover_arg_dest_storage);
	if($status eq $RTN_DB_PROTECTED) {
	    
	    &recover_set_state("recover_st_protected");
	} 
	else {
	    
	    &recover_error("db protect failed \"$recover_arg_dest_storage\"\n");
	    $ret_status = $RTN_NOT_OK;
	    last CLEANUP_RESTORE_STORAGE;
	} 
    }


    $ret_status = $RTN_OK;

  } # CLEANUP_RESTORE_STORAGE Always return here

    &recover_restore_cur_function();
    return $ret_status;
  
}



#
# recover_enable_replica
# Inputs:
# Outputs:
#   None
# Returns:
#   None
# References:
# Modifies:
# Calls:
# Description:
#   The data is in place in $recover_arg_dest_storage. Register the replica
#   first. When this is done the schema file can me moved into place and the
#   lock checked. 
#
sub recover_enable_replica {

    local($status);                   # General return status 
    local($fstr);                     # General format string
    local($ret_status);               # Status return at end of CLEANUP block
    local(@exclude);                  # Lock excludes
    local(@cmdbuf);
    local($command);

    &recover_set_cur_function("recover_enable_replica");
    $ret_status = $RTN_NO_STATUS_SET; 

CLEANUP_ENABLE_REPLICA: { # Always return at end of this block.

    #
    # Some of the operations to follow are required to be run as whoever 
    # owns the storage area. Set this ownership to root for now. The 
    # ownership will be set properly when &recover_enable_replica returns.
    # Set $recover_reset_owner so cleanup knows to put it right if we
    # exit prematurely.
    #
    $status = &mdep_set_ownership($recover_arg_dest_storage);

    if($status eq $RTN_OK) {
	
	$recover_cl_reset_owner = 1;
    }

    if(!&recover_st_chk("recover_st_new_tag")) {
	print "Making tag for storage $recover_arg_dest_storage...";
	$status = &mdep_mktag($recover_arg_vob_tag,
			      $recover_arg_dest_storage);

	if($status ne $RTN_MKTAG_OK) {
	    
	    &recover_error("Could not make new vob tag\n");
	    $ret_status = $RTN_NOT_OK;
	    last CLEANUP_ENABLE_REPLICA;
	}
	&recover_set_state("recover_st_new_tag");
	print "mktag complete\n";
    }


    # 
    # Unprotect the database. The database must be protected by cleanups we
    # write. 
    # 
    if((&recover_st_chk("recover_st_protected"))) {

	$status = &recover_unprotect_data_base($recover_arg_dest_storage);

	if($status ne $RTN_DB_UNPROTECTED) {

	    &recover_error("Unprotect operation failed \"$recover_arg_dest_storage\".\n");
	    $ret_status = $RTN_NOT_OK;
	    last CLEANUP_ENABLE_REPLICA;
	}
	&recover_unset_state("recover_st_protected");
    }
    
    
    if(!&recover_st_chk("recover_st_new_registration")) {
	print "Registering $recover_arg_dest_storage...";
	$status = &mdep_register_vob_object($recover_arg_dest_storage);
	if($status ne $RTN_REGISTER_OK) {
	    
	    &recover_error("Could not register vob\n");
	    $ret_status = $RTN_NOT_OK;
	    last CLEANUP_ENABLE_REPLICA;
	}
        &recover_set_state("recover_st_new_registration");
	print "register complete\n";
    }

    if(!$CLEARCASE_LIGHT) {
        mdep_mount($recover_arg_vob_tag);
    }


    # 
    # We don't want to trust the old state if this is a restart. Make
    # sure the vob is locked.
    #
    if(!&recover_st_chk("recover_st_lock_checked") &&
       !&mdep_is_locked($recover_arg_vob_tag)) {

        local(@excludes);
	local($replace) = $MDEP_REPLACE_LOCK;

        $comment = "vob_restore: vob backed up unlocked. Locked now.";
	@excludes = ();
	$replace = $MDEP_REPLACE_LOCK;
	$status = &mdep_lock_vob(*recover_arg_vob_tag,
				 *recover_arg_dest_storage,
				 *replace,*comment,*excludes);
	
	#
	# This is the first time we've looked at the lock and it was found
	# unlocked. If a dbcheck hasn't been done mark it as being needed.
	#
	if(!&recover_st_chk("recover_st_dbcheck_done")) {
	    
	    #
	    # Oh,Oh. Mark it. Until this run or a subsequent run logs
	    # a "recover_st_dbcheck_done", the dbcheck will be required
	    #
	    &recover_set_state("recover_st_dbcheck_needed");
	    
	}
	    
	&recover_set_state("recover_st_lock_checked");
    }
       
    #
    # If recover_dbcheck_needed is set and recover_dbcheck_needed is not
    # force a dbcheck.
    #
    if(&recover_st_chk("recover_st_dbcheck_needed") &&
       (!&recover_st_chk("recover_st_dbcheck_done"))) {

        #
	# Be nice and give the user a chance to do it later. 
	#
	$reply = &mdep_prompt($RECOVER_P_DBCHECK_REQUIRED,
				   $MDEP_YES_QUIT,
				   $MDEP_YES,
				   $MDEP_NO_HELP);
	if(lc($reply) eq $MDEP_QUIT) {
	    
	    $ret_status = $RTN_NOT_OK;
	    last CLEANUP_ENABLE_REPLICA;
	}
	print "\nPerforming database check. This may take some time...";
	$status = &mdep_dodbcheck($recover_arg_dest_storage);

	if($status ne $RTN_DBCHECK_OK) {
	    
	    &recover_error("Dbcheck failed\n");
	    $ret_status = $RTN_NOT_OK;
	    last CLEANUP_ENABLE_REPLICA;
	}
	print "checked clean \n";	
	&recover_set_state("recover_st_dbcheck_done");
	&recover_unset_state("recover_st_dbcheck_needed");
    }
       
    if($recover_cl_reset_owner) {
	
	&mdep_reset_ownership($recover_arg_vob_tag,$recover_arg_dest_storage);
	$recover_cl_reset_owner = 0;
    }
    $command = "$MDEP_CLEARTOOL describe -long  vob:$recover_arg_vob_tag";
    if(&mdep_command_s(*command,*cmdbuf)) {
 
        $ret_status = 1;
        last CLEANUP_ENABLE_REPLICA;
    }
 
    #
    # Set each of the rgy globals.
    #
    foreach $line (@cmdbuf) {
	if($line =~ /master replica:/) {
            $recover_replicated = 1;
            &recover_set_state("recover_replicated");
            last;
	}
    }

       
    $ret_status = $RTN_OK

  } # CLEANUP_ENABLE_REPLICA Always return here
  &recover_restore_cur_function();
  return $ret_status;   

}        


sub recover_cleanup {

    local($status);           # General status word.
    local(@excludes);          # Lock excludes
    local($replace);
    local($comment);
    local($state_dir);

    &recover_set_cur_function("recover_cleanup");
    $ret_status = $RTN_NO_STATUS_SET; 

    
    if($recover_cl_start_clearcase) {

	#
	# Start clearcase.
	#
	print "Restarting Clearcase...";
		if($MDEP_MACHINE eq 'NT') {
	    &mdep_start_clearcase_nt();
	}
	else {
	    &mdep_start_clearcase();
	}
if(&mdep_is_clearcase_up()) {
	    print "Clearcase up\n";
	}
    }

    @excludes = ();
    $replace = $MDEP_REPLACE_LOCK;
    $comment = "vob_restore locking on error exit";
    &mdep_lock_vob(*recover_arg_vob_tag,
		   *recover_arg_dest_storage,
		   *replace,*comment,*excludes);

    if($recover_do_save_state) {
	&recover_write_state();
	$fstr = sprintf($RECOVER_STATE_FILE_LOCATION,$recover_state_dir);
	print $fstr;
    }

    exit(1);

}

sub recover_alldone {

    local(@excludes);
    local($replace);
    local($comment);
    $tstate = $recover_arg_temp_storage.$MDEP_SEP.$recover_state_file;
    $dstate = $recover_arg_dest_storage.$MDEP_SEP.$recover_state_file;

    @excludes = ();
    $replace = $MDEP_REPLACE_LOCK;
    $comment = "vob_restore locking on clean exit";
    &mdep_lock_vob(*recover_arg_vob_tag,
		   *recover_arg_dest_storage,
		   *replace,*comment,*excludes);
    
    if(!&recover_st_chk("recover_checkvob_done")) {
 
        if(&recover_st_chk("recover_replicated")) {
 
            print $RECOVER_CHECKVOB_NOT_RUN_REPLICATED;
        }
        else {
            print $RECOVER_CHECKVOB_NOT_RUN;
        }
 
        &recover_write_state();
        $fstr = sprintf($RECOVER_STATE_FILE_LOCATION,$recover_state_dir);
	print $fstr;
        exit(1);
    }
    else {

        if(&recover_st_chk("recover_replicated")) {
 
            print $RECOVER_CHECKVOB_CLEAN_REPLICATED;
        }
        else {
            print $RECOVER_CHECKVOB_CLEAN;
        }
    }

    if(-e $tstate) {
 
        &mdep_rmfile($tstate);
    }
 
    if(-e $dstate) {
 
        &mdep_rmfile($dstate);
    }
 
    exit(0);
 
}

sub recover_checkvob_command {

    local(*command,*view,*qsource,*qderived,*qcleartxt) = @_;
    local($reply);

    if($qsource) {

	$reply = &mdep_prompt($RECOVER_P_CHECKVOB_CHECK_SOURCE,
			      $MDEP_YES_OR_NO,
			      $MDEP_YES,
			      $MDEP_NO_HELP);
	
	if(lc($reply) eq $MDEP_YES) {
	    
	    $command .= ' -source';
	}
    }
    
    if($qderived) {

	$reply = &mdep_prompt($RECOVER_P_CHECKVOB_CHECK_DERIVED,
			      $MDEP_YES_OR_NO,
			      $MDEP_YES,
			      $MDEP_NO_HELP);
	
	if(lc($reply) eq $MDEP_YES) {
	    
	    $command .= ' -derived';
	}
    }
    
    if($qcleartxt) {

	$reply = &mdep_prompt($RECOVER_P_CHECKVOB_CHECK_CLEARTXT,
			      $MDEP_YES_OR_NO,
			      $MDEP_YES,
			      $MDEP_NO_HELP);
	
	if(lc($reply) eq $MDEP_YES) {
	    
	    $command .= ' -cleartext';
	}
    }
    
    if(!$view) {

	$view = &mdep_prompt($RECOVER_P_GET_CHECKVOB_TAG,
			    $MDEP_ANY_RESPONSE,
			    $MDEP_NO_DEFAULT,
			    $RECOVER_H_GET_TAG);
	
    }
	
    $view =~ s/\s//g;
    
    if($view) {
	
	$command .= " -view $view";
    }

    $command .= " \"$recover_arg_dest_storage\"";

}

sub recover_print_checkvob_record {

    local(*record) = @_;

    print "INIT_NDATA_OBJS: $record{$RECOVER_PL_INIT_NDATA_OBJS}\n";
    print "INIT_MISPROT_OBJS: $record{$RECOVER_PL_INIT_MISPROT_OBJS}\n";
    print "FIN_NDATA_OBJS: $record{$RECOVER_PL_FIN_NDATA_OBJS}\n";
    print "FIN_MISPROTECTED_OBJS: $record{$RECOVER_PL_FIN_MISPROTECTED_OBJS}\n";
    print "FIX_DATA_LOSS_OBJ: $record{$RECOVER_PL_FIX_DATA_LOSS_OBJ}\n";
    print "FIX_DATA_LOSS_ITEMS: $record{$RECOVER_PL_FIX_DATA_LOSS_ITEMS}\n";
}

    
sub recover_print_checkvob_initials {

    local(*serrors,*derrors,*cerrors) = @_;
    local($source_c);
    local($source_m);
    local($derived_c);
    local($derived_m);
    local($cleartxt_c);
    local($cleartxt_m);
    local($fstr);
    
    $source_c = $recover_source_record{$RECOVER_PL_INIT_NDATA_OBJS};
    $source_m = $recover_source_record{$RECOVER_PL_INIT_MISPROT_OBJS};
    $derived_c = $recover_derived_record{$RECOVER_PL_INIT_NDATA_OBJS};
    $derived_m = $recover_derived_record{$RECOVER_PL_INIT_MISPROT_OBJS};
    $cleartxt_c = $recover_cleartxt_record{$RECOVER_PL_INIT_NDATA_OBJS};
    $cleartxt_m = $recover_cleartxt_record{$RECOVER_PL_INIT_MISPROT_OBJS};

    $serrors = 0;
    $derrors = 0;
    $cerrors = 0;
    
    $fstr = sprintf($RECOVER_E_INIT_C_CNT,$source_c,'source');
    print $fstr;
    $serrors += $source_c;
    
    $fstr = sprintf($RECOVER_E_INIT_C_CNT,$derived_c,'derived object');
    print $fstr;
    $derrors += $derived_c;
    
    $fstr = sprintf($RECOVER_E_INIT_M_CNT,$source_m,'source');
    print $fstr;
    $serrors += $source_m;


    $fstr = sprintf($RECOVER_E_INIT_M_CNT,$derived_m,'derived object');
    print $fstr;
    $derrors += $derived_m;

    print $RECOVER_CLEARTXT_NOT_CHECKED;        
}

sub recover_print_checkvob_final {

    local(*serrors,*derrors,*cerrors) = @_;
    local($fstr);
    local($source_fnc);
    local($source_fnm);
    local($source_fxlo);
    local($source_fxli);
    local($derived_fnc);
    local($derived_fnm);
    local($derived_fxlo);
    local($derived_fxli);
    local($cleartxt_fnc);
    local($cleartxt_fnm);

    $source_fnc = $recover_source_record{$RECOVER_PL_FIN_NDATA_OBJS};
    $source_fnm = $recover_source_record{$RECOVER_PL_FIN_MISPROT_OBJS};
    $source_fxlo = $recover_source_record{$RECOVER_PL_FIX_DATA_LOSS_OBJ};
    $source_fxli = $recover_source_record{$RECOVER_PL_FIX_DATA_LOSS_ITEMS};
    $derived_fnc = $recover_derived_record{$RECOVER_PL_FIN_NDATA_OBJS};
    $derived_fnm = $recover_derived_record{$RECOVER_PL_FIN_MISPROT_OBJS};
    $derived_fxlo = $recover_derived_record{$RECOVER_PL_FIX_DATA_LOSS_OBJ};
    $derived_fxli = $recover_derived_record{$RECOVER_PL_FIX_DATA_LOSS_ITEMS};
    $cleartxt_fnc = $recover_cleartxt_record{$RECOVER_PL_FIN_NDATA_OBJS};
    $cleartxt_fnm = $recover_cleartxt_record{$RECOVER_PL_FIN_MISPROT_OBJS};

    $serrors = 0;
    $derrors = 0;
    $cerrors = 0;

    $fstr = sprintf($RECOVER_FN_C_OBJS,$source_fnc,'source');
    print $fstr;
    $serrors += $source_fnc;
    
    $fstr = sprintf($RECOVER_FN_M_OBJS,$source_fnm,'source');
    print $fstr;
    $serrors += $source_fnm;
    
    $fstr = sprintf($RECOVER_FX_LO_SRC,$source_fxlo);
    print $fstr;
    
    $fstr = sprintf($RECOVER_FX_LI_SRC,$source_fxli);
    print $fstr;
    
    $fstr = sprintf($RECOVER_FN_C_OBJS,$derived_fnc,'derived object');
    print $fstr;
    $derrors += $derived_fnc;
    
    $fstr = sprintf($RECOVER_FN_M_OBJS,$derived_fnm,'derived object');
    print $fstr;
    $derrors += $derived_fnm;
    
    $fstr = sprintf($RECOVER_FX_LO_DO,$derived_fxlo);
    print $fstr;
    
    $fstr = sprintf($RECOVER_FX_LI_DO,$derived_fxli);
    print $fstr;

    print $RECOVER_CLEARTXT_NOT_CHECKED;
}
    
sub recover_check_the_vob {

    local($summary);            # The summary file
    local($reply);              # Reply buffer
    local($myid);               # My login id
    local($status);             # General status
    local($fstr);               # General format buffer
    local($summary);            # Summary file tmp name
    local($checkvob_summary);   # Summary file name
    local($checkvob_date);      # Holds date
    local($checkvob_log);       # Log directory for checkvob
    local($summary_command);    # Checkvob summary command
    local($error);              # Error count
    local($fix);                # Fix summary file tmp name
    local($checkvob_fix);       # Fix summary file name
    local($fix_command);        # Checkvob fix command
    local(@excludes);           # Lock excludes
    local($replace);
    local($comment);
    local($serrors);            # Source container errors
    local($derrors);            # Derived object container errors
    local($cerrors);            # Clear text container errors
    local($qry_source);
    local($qry_derived);
    local($qry_cleartxt);
    local($view_tag) = "";      # 

    &recover_set_cur_function("recover_check_the_vob");

CLEANUP_CHECK_THE_VOB: { # Always return at end of this block.

    $reply = &mdep_prompt($RECOVER_P_RUN_CHECKVOB,
			  $MDEP_YES_HELP_QUIT,
			  $MDEP_YES,
			  $RECOVER_H_RUN_CHECKVOB);
    
    
    if(lc($reply) eq $MDEP_QUIT) {
	
	$ret_status = 1;
	last CLEANUP_CHECK_THE_VOB;
    }

    #
    # It should be locked still. Leave it locked except for root who we
    # already verified is running this script but on NT its not as clear.
    # call a machine dependent function to get our login id.
    #
    $myid = &mdep_get_login_id();

    if(! $myid) {
	&recover_error("Could not determine login id\n");
	$ret_status = -1;
	last CLEANUP_CHECK_THE_VOB;
    }



    $summary = &recover_st_chk("recover_checkvob_summary");
    
    if($summary &&
       (-s $summary)) {
	
	$fstr = sprintf($RECOVER_P_CHECKVOB_REUSE_SUMMARY,$summary);
	
	$reply = &mdep_prompt($fstr,
			      $MDEP_YES_NO_OR_QUIT,
			      $MDEP_YES,
			      $RECOVER_H_CHECKVOB_REUSE_SUMMARY);
	if(lc($reply) eq $MDEP_QUIT) {
	    
	    $ret_status = 1;
	    last CLEANUP_CHECK_THE_VOB;
	}
	
	if(lc($reply) eq $MDEP_YES) {
	    
	    $checkvob_summary = $summary;
	}
	else {

	    &recover_unset_state("recover_st_checkvob_summary");
	}
    }
    
    if($checkvob_summary eq "") {
	
	$checkvob_date = &mdep_date_fmt_time_long(time());
	$checkvob_log = $recover_state_dir.$MDEP_SEP.'checkvob_sum.'.$checkvob_date;
	
	$reply = &mdep_prompt($RECOVER_P_CHECKVOB_RUN_SUMMARY,
			      $MDEP_YES_NO_QUIT_HELP,
			      $MDEP_YES,
			      $RECOVER_H_CHECKVOB_RUN_SUMMARY);
	
	if(lc($reply) eq $MDEP_QUIT) {
	    
	    $ret_status = 1;
	    last CLEANUP_CHECK_THE_VOB;
	}
	
	if(lc($reply) eq $MDEP_YES) {
	    
	    $summary_command = $MDEP_CHECKVOB;
	    $summary_command .= " -log $checkvob_log";
	    $summary_command .= ' -pool';
	    
	    $qry_source = 1;
	    $qry_derived = 1;
	    $qry_cleartxt = 1;

	    &recover_checkvob_command(*summary_command,*view_tag,*qry_source,
				      *qry_derived,*qry_cleartxt);
	    
	    $fstr = sprintf($RECOVER_P_CHECKVOB_SUMMARY_GO,
			    $checkvob_log);
	    
	    $reply = &mdep_prompt($fstr,
				  $MDEP_QUIT_RETURN_TO_CONTINUE,
				  $MDEP_NONE,
				  $MDEP_NO_HELP);

	    if(lc($reply) eq $MDEP_QUIT) {

		$ret_status = 1;
		last CLEANUP_CHECK_THE_VOB;
	    }
	    $status = system($summary_command); 
	    
	    if($status) {

		&recover_error($RECOVER_E_CHECKVOB_SUMMARY_FAILED);
		$ret_status = -1;
		last CLEANUP_CHECK_THE_VOB;
	    }
	    
	    $checkvob_summary = $checkvob_log.$MDEP_SEP.'.summary';
	    &recover_unset_state("recover_st_checkvob_summary");
	    &recover_unset_state("recover_st_checkvob_fix");
	    &recover_set_state(
		       "recover_checkvob_summary=$checkvob_summary");
	}
    }
    
    &recover_init_pools();

    if($checkvob_summary) {

	$status = &recover_parse_checkvob_summary(*checkvob_summary);
	if($status ne $RTN_OK) {
	    
	    $fstr = sprintf($RECOVER_PARSE_CHECKVOB_SUMMARY_FAILED,
			    $checkvob_summary);
	    &recover_error($fstr);
	    $ret_status = -1;
	    last CLEANUP_CHECK_THE_VOB;
	}
	
	$fstr = sprintf($RECOVER_CHECKVOB_SUMMARY_RESULTS,$checkvob_summary);
	print $fstr
        &recover_print_checkvob_initials(*serrors,*derrors,*cerrors);
        print $RECOVER_MORE_SUMMARY_DETAILS;

	$errors = $serrors + $derrors + $cerrors;

	if(!$errors) {
	    
	    $ret_status = 0;
	    last CLEANUP_CHECK_THE_VOB;
	}
    }
	
    $fix = &recover_st_chk("recover_checkvob_fix");
    
    if(($fix) &&
       (-s $fix)) {
	
	$fstr = sprintf($RECOVER_P_CHECKVOB_REUSE_FIX,$fix);
	
	$reply = &mdep_prompt($fstr,
			      $MDEP_YES_NO_QUIT_HELP,
			      $MDEP_YES,
			      $RECOVER_H_CHECKVOB_REUSE_FIX);
	if(lc($reply) eq $MDEP_QUIT) {
	    
	    $ret_status = 1;
	    last CLEANUP_CHECK_THE_VOB;
	}
	
	if(lc($reply) eq $MDEP_YES) {
	    
	    $checkvob_fix = $fix;
	}
	else {

	    &recover_unset_state("recover_checkvob_fix");
	}
    }
	
    if(!$checkvob_fix) {
	
	$checkvob_date = &mdep_date_fmt_time_long(time());
	$checkvob_log = $recover_state_dir.$MDEP_SEP.'checkvob_fix.'.$checkvob_date;
	
	$reply = &mdep_prompt($RECOVER_P_CHECKVOB_RUN_FIX,
			      $MDEP_YES_NO_OR_QUIT,
			      $MDEP_YES,
			      $RECOVER_H_CHECKVOB_RUN_FIX);
	
	if(lc($reply) eq $MDEP_QUIT) {
	    
            $ret_status = 1;
            last CLEANUP_CHECK_THE_VOB;

	}
	
	if(lc($reply) eq $MDEP_YES) {
	    
	    $fix_command = $MDEP_CHECKVOB;
	    $fix_command .= " -fix -force -log $checkvob_log";
	    $fix_command .= ' -pool';
	    $fix_command .= ' -ignore';

	    $qry_source = 1;
	    $qry_derived = 1;
	    $qry_cleartxt = 1;

	    if($checkvob_summary) {

		if(!$serrors) {

		    $qry_source = 0;
		}
		if(!$derrors) {
		    
		    $qry_derived = 0;
		}
		if(!$cerrors) {

		    $qry_cleartxt = 0;
		}
	    }

	    &recover_checkvob_command(*fix_command,*view_tag,*qry_source,
				      *qry_derived,*qry_cleartxt);
    
	    $fstr = sprintf($RECOVER_P_CHECKVOB_FIX_GO,
			    $checkvob_log);
	    
	    $reply = &mdep_prompt($fstr,
				  $MDEP_YES_QUIT,
				  $MDEP_YES,
				  $MDEP_NO_HELP);

	    if(lc($reply) eq $MDEP_QUIT) {

                $ret_status = 1;
                last CLEANUP_CHECK_THE_VOB;
	    }

	    $fstr = sprintf($RECOVER_P_NUSERS_OK,$myid);
	    $reply = &mdep_prompt($fstr,
				  $MDEP_QUIT_RETURN_TO_CONTINUE,
				  $MDEP_NONE,
				  $MDEP_NO_HELP);
	    if(lc($reply) eq $MDEP_QUIT) {
		&recover_cleanup();
	    }
	    
	    #
	    # Lock for all but me.
	    #
	    @excludes = ("$myid");
	    $replace = $MDEP_REPLACE_LOCK;
	    $comment = "vob_restore locking for checkvob run";
	    $status = &mdep_lock_vob(*recover_arg_vob_tag,
				     *recover_arg_dest_storage,
				     *replace,*comment,*excludes);
	    
	    if($status ne $RTN_LOCK_OK) {
		$fstr = "Could not lock the vob to run checkvob.\n";
		&recover_error($fstr);
		$ret_status = -1;
		last CLEANUP_CHECK_THE_VOB;
	    }
            print "fix_command: $fix_command\n";
	    $status = system("$fix_command"); 

	    if($status) {
		
                if(&recover_st_chk("recover_replicated")) {
                    $fstr = sprintf($RECOVER_E_CHECKVOB_FIX_FAILED_REPLICATED,
                                    $checkvob_log);
                }
                else {
                    $fstr = sprintf($RECOVER_E_CHECKVOB_FIX_FAILED,
                                    $checkvob_log);
                }
                &recover_error($fstr);
                $ret_status = -1;
                last CLEANUP_CHECK_THE_VOB;
	    }
	    
	    $checkvob_fix = $checkvob_log.$MDEP_SEP.'.summary';
	    &recover_unset_state("recover_checkvob_fix");
	    &recover_set_state("recover_checkvob_fix=$checkvob_fix");
	}
    }
	
    &recover_init_pools();

    if($checkvob_fix) {

	$status = &recover_parse_checkvob_summary(*checkvob_fix);
	if($status ne $RTN_OK) {
	   
	   if(-d $checkvob_log) { 
	   	$fstr = sprintf($RECOVER_PARSE_CHECKVOB_FIX_FAILED,
			    $checkvob_fix);
	   }
	   else {
		$fstr = sprintf($RECOVER_PARSE_CHECKVOB_FIX_FAILED_NO_DIR,
			    $checkvob_fix);
	   }
		
	    &recover_error($fstr);
	    $ret_status = -1;
	    &recover_unset_state("recover_checkvob_fix");
	    last CLEANUP_CHECK_THE_VOB;
	}
    
	$fstr = sprintf($RECOVER_FIX_RESULTS,$checkvob_summary);
	print $fstr
        &recover_print_checkvob_final(*serrors,*derrors,*cerrors);

	$errors = $serrors + $derrors + $cerrors;

	if(!$errors) {
	    
#	    print $RECOVER_CHECKVOB_CLEAN;
	    $ret_status = 0;
	    last CLEANUP_CHECK_THE_VOB;
	}
	else {
            $fstr = sprintf($RECOVER_CHECKVOB_FIX_LEFT_ERRORS,
                                $checkvob_log);
	    &recover_error($fstr);
            $ret_status = -1;
            last CLEANUP_CHECK_THE_VOB;
	}
    }
    else {

	$ret_status = 1;
	last CLEANUP_CHECK_THE_VOB;
    }

    $ret_status = 0;

} # CLEANUP_CHECK_THE_VOB Always return here

    &recover_restore_cur_function();
    return $ret_status;
}

sub recover_init_pools {

    $recover_source_record{$RECOVER_PL_INIT_NDATA_OBJS} = 0;
    $recover_source_record{$RECOVER_PL_INIT_MISPROT_OBJS} = 0;
    $recover_source_record{$RECOVER_PL_FIN_NDATA_OBJS} = 0;
    $recover_source_record{$RECOVER_PL_FIN_MISPROTECTED_OBJS} = 0;
    $recover_source_record{$RECOVER_PL_FIX_DATA_LOSS_OBJ} = 0;
    $recover_source_record{$RECOVER_PL_FIX_DATA_LOSS_ITEMS} = 0;

    $recover_derived_record{$RECOVER_PL_INIT_NDATA_OBJS} = 0;
    $recover_derived_record{$RECOVER_PL_INIT_MISPROT_OBJS} = 0;
    $recover_derived_record{$RECOVER_PL_FIN_NDATA_OBJS} = 0;
    $recover_derived_record{$RECOVER_PL_FIN_MISPROTECTED_OBJS} = 0;
    $recover_derived_record{$RECOVER_PL_FIX_DATA_LOSS_OBJ} = 0;
    $recover_derived_record{$RECOVER_PL_FIX_DATA_LOSS_ITEMS} = 0;

    $recover_cleartxt_record{$RECOVER_PL_INIT_NDATA_OBJS} = 0;
    $recover_cleartxt_record{$RECOVER_PL_INIT_MISPROT_OBJS} = 0;
    $recover_cleartxt_record{$RECOVER_PL_FIN_NDATA_OBJS} = 0;
    $recover_cleartxt_record{$RECOVER_PL_FIN_MISPROTECTED_OBJS} = 0;
    $recover_cleartxt_record{$RECOVER_PL_FIX_DATA_LOSS_OBJ} = 0;
    $recover_cleartxt_record{$RECOVER_PL_FIX_DATA_LOSS_ITEMS} = 0;

}



#
# recover_parse_checkvob_summary
# Inputs:
#   $summary 
#     .summary path
# Outputs:
#   None
# Returns:
#   None
# References:
# Modifies:
# Calls:
# Description:
#   Parse checkvob's .summary output.
#
sub recover_parse_checkvob_summary {

    local(*summary) = @_;
    local(@pool_fields);
    local($field);
    local($known_field);
    local($ret_status);
    local($fstr);
    local($line);


    &recover_set_cur_function("recover_parse_checkvob_summary");

CLEANUP_PARSE_CHECKVOB_SUMARY: { # Always return at end of this block.

    unless(open(SUMMARY_FILE,$summary)) {
	$fstr = sprintf($RECOVER_E_SUMMARY_OPEN_FAILED,$summary);
	&recover_error($fstr);
	$ret_status = $RTN_NOT_OK;
	last CLEANUP_PARSE_CHECKVOB_SUMARY;
    }
    

    foreach $line (<SUMMARY_FILE>) {

	if($line =~ /PoolKind=source/) {
	    
	    @pool_fields = split(/\s/,$line);
	    
	    foreach $known_field (@recover_known_pool_fields) {
		
		foreach $field (@pool_fields) {
		    
		    if($field =~ /$known_field=(.*)/) {

			$recover_source_record{$known_field} = $1;
		    }
		}
	    }
	}
	elsif($line =~ /PoolKind=derived/) {

	    @pool_fields = split(/\s/,$line);
	    
	    foreach $known_field (@recover_known_pool_fields) {
	
		foreach $field (@pool_fields) {
		    
		    if($field =~ /$known_field=(.*)/) {
			
			$recover_derived_record{$known_field} = $1;
		    }
		}
	    }
	}
	elsif($line =~ /PoolKind=cleartext/) {

	    @pool_fields = split(/\s/,$line);
    
	    foreach $known_field (@recover_known_pool_fields) {
	
		foreach $field (@pool_fields) {
		    
		    if($field =~ /$known_field=(.*)/) {
			
			$recover_cleartxt_record{$known_field} = $1;
		    }
		}
	    }
	}
    }

    $ret_status = $RTN_OK;

  } # CLEANUP_PARSE_CHECKVOB_SUMARY Always return here

    &recover_restore_cur_function();
    close(SUMMARY_FILE);

    return $ret_status;

}

sub recover_set_inputs {

    $recover_arg_dest_storage = &recover_st_chk("recover_arg_dest_storage");
    $recover_arg_temp_storage = &recover_st_chk("recover_arg_temp_storage");
    $recover_arg_snap_storage = &recover_st_chk("recover_arg_snap_storage");
    $recover_rgy_replica_uuid = &recover_st_chk("recover_rgy_replica_uuid");
    $recover_rgy_family_uuid = 	&recover_st_chk("recover_rgy_family_uuid");
    $recover_rgy_storage = &recover_st_chk("recover_rgy_storage");
    $recover_rgy_host = &recover_st_chk("recover_rgy_host");
    $recover_scenario =  &recover_st_chk("recover_scenario");
    $recover_replicated = &recover_st_chk("recover_replicated");
    $recover_arg_back_storage = &recover_st_chk("recover_arg_back_storage");
    $recover_backup_equals_registered = 
        &recover_st_chk("recover_backup_equals_registered");
    $recover_backup_overwrite_done = 
	&recover_st_chk("recover_backup_overwrite_done");
    $recover_registry_save_file = &recover_st_chk("recover_registry_save_file");
}

sub recover_clear_inputs {

    $recover_rgy_replica_uuid = "";
    $recover_rgy_family_uuid = "";
    $recover_rgy_storage = "";
    $recover_rgy_host = "";
    $recover_arg_dest_storage = "";
    $recover_arg_temp_storage = "";
    $recover_arg_snap_storage = "";
    $recover_scenario = -1;
    $recover_replicated = 0;
    $recover_arg_bask_storage = "";
    $recover_backup_overwrite_done = 0;
    $recover_registry_save_file = "";
}

sub recover_unregister {

    local($reply);
    local($status);
    local($fstr);
    local($ret_status);

CLEANUP_UNREGISTER: {

    if($recover_scenario == $RECOVER_SC_DOWN) {
	
	if(!&recover_st_chk("recover_other_unregisters_qry")) {
	    $reply = &mdep_prompt($RECOVER_P_OTHER_UNREGISTERS,
				  $MDEP_QUIT_RETURN_TO_CONTINUE,
				  $MDEP_NONE,
				  $MDEP_NO_HELP); 
	    if(lc($reply) eq $MDEP_QUIT) { 
		
		$ret_status = 1;
		last CLEANUP_UNREGISTER;
		
	    }
	    &recover_set_state("recover_other_unregisters_qry");
	}
	last CLEANUP_UNREGISTER;
    }

    if(!&recover_st_chk("recover_st_did_unregister")) {
    
	#
	# Check with the user for verification. Allow quit for now.
	#
	$reply = &mdep_prompt($RECOVER_P_CCASE_MADE_UNAVAILABLE,
			      $MDEP_QUIT_RETURN_TO_CONTINUE,
			      $MDEP_NONE,
			      $RECOVER_H_CCASE_MADE_UNAVAILABLE);
	if(lc($reply) eq $MDEP_QUIT) {
	    $ret_status = 1;
	    last CLEANUP_UNREGISTER;
	}

	if($recover_registered && 
	   !&recover_st_chk("recover_saved_registry_state")) {

	    $recover_registry_save_file = 
		&recover_st_chk("recover_registry_save_file");
	    $fstr_p = sprintf($RECOVER_P_SAVE_RGY_INFO,
			      $recover_registry_save_file);
	    $fstr_h = sprintf($RECOVER_H_SAVE_RGY_INFO,
			      $recover_registry_save_file);
	    $reply = &mdep_prompt($fstr_p,
				  $MDEP_QUIT_RETURN_TO_CONTINUE,
				  $MDEP_NONE,
				  $fstr_h);
	    if(lc($reply) eq $MDEP_QUIT) {
		$ret_status = 1;
		last CLEANUP_UNREGISTER;
	    }
            if($CLEARCASE_LIGHT) {
                my($region) = "";
            }
            else {
                if($MDEP_MACHINE eq 'NT') {
                    my($region) = "-region *";
                } else {
                    my($region) = "-region \'*\'";
                }
            }
	    if($MDEP_MACHINE eq 'NT') {
	    $fstr = sprintf(
		    "$MDEP_CLEARTOOL lsvob -long $region -storage \"%s\" > \"%s\"",
		    $recover_rgy_storage,$recover_registry_save_file);
            }
            else {
	    $fstr = sprintf(
		    "$MDEP_CLEARTOOL lsvob -long $region -storage %s > %s",
		    $recover_rgy_storage,$recover_registry_save_file);
            }
	    #
	    # Save NLSPATH and set to null for English
	    #
	    $NLSPATH = $ENV{NLSPATH}; 
	    $ENV{NLSPATH} = "";
	    $status = system($fstr);
	    #
	    # Now restore NLSPATH 
	    #
	    $ENV{NLSPATH} = $NLSPATH; # Reset NLSPATH
	    if($status || 
	       (! -s $recover_registry_save_file)) {
                
		$fstr = sprintf($RECOVER_N_SAVE_RGY_TAG,
				$recover_arg_dest_storage,
				$recover_arg_vob_tag);
		print $fstr;
		$fstr = sprintf(
		    "$MDEP_CLEARTOOL lsvob -long $region %s > %s",
		    $recover_arg_vob_tag,$recover_registry_save_file);
		#
		# Save NLSPATH and set to null for English
		#
		$NLSPATH = $ENV{NLSPATH}; 
		$ENV{NLSPATH} = "";
		$status = system($fstr);
		#
		# Now restore NLSPATH 
		#
		$ENV{NLSPATH} = $NLSPATH; # Reset NLSPATH

		if(!$status) {
		    $reply = &mdep_prompt($RECOVER_P_SAVE_RGY_TAG_OK,
					  $MDEP_QUIT_RETURN_TO_CONTINUE,
					  $MDEP_ANY_RESPONSE,
					  $MDEP_NONE,
					  $MDEP_NO_HELP);
		}
		else {
		    $reply = &mdep_prompt($RECOVER_P_SAVE_RGY_TAG_FAILED,
					  $MDEP_QUIT_RETURN_TO_CONTINUE,
					  $MDEP_NONE,
					  $MDEP_NO_HELP);
		}
		
		if(lc($reply) eq $MDEP_QUIT) {
		    $ret_status = 1;
		    last CLEANUP_UNREGISTER;
		}
	    }
	}

	#
	# If its mounted unmount it. Pay no attention to the status. It may not
	# have been mounted.
	#
        if(!$CLEARCASE_LIGHT) {
	    &mdep_umount($recover_arg_vob_tag);
        }
	
	#
	# If the original tag hasn't been removed yet remove it. If its not 
	# the first time, checks have already been done above to verify that if
	# a vob currently exists with the same tag, its this one pointing at 
	# its new storage, so don't remove it. 
	# 
	if(!&recover_st_chk("recover_st_orig_tag_removed")) {
	    
	    print "Removing vob tag $recover_arg_vob_tag...\n";
	    $status = &Rmtag($recover_arg_vob_tag);
	    
	    if($status eq $RTN_RMTAG_FAILED) {
		
		&recover_error("Could not remove vob tag\n");
		$ret_status = 1;
		last CLEANUP_UNREGISTER;
	    }
	     print "rmtag complete\n";
	}
	&recover_set_state("recover_st_orig_tag_removed");
	
	
	#
	# If the original vob hasn't been unregistered yet unregister it. 
	#
	if(!&recover_st_chk("recover_st_orig_unregistered")) {
	    
	    print "Unregistering $recover_rgy_storage...";
	    $status = &mdep_unregister_vob_object($recover_rgy_replica_uuid);
	    
	    if($status eq $RTN_UNREGISTER_FAILED) {
		
		&recover_error("Unregister failed\n");
		$ret_status = 1;
		last CLEANUP_UNREGISTER;
	    }
	    print "unregister complete\n";
	}
	&recover_set_state("recover_st_orig_unregistered");
	
	&recover_set_state("recover_st_did_unregister");
	
	$ret_status = 0;
    }
  } #CLEANUP_UNREGISTER
    
    return $ret_status;
}

sub recover_stop_clearcase {

    local($status);
    local($ret_status);

CLEANUP_STOP_CLEARCASE: {
    

    $status = &mdep_is_clearcase_up();
    if($status < 0) {
	
	&recover_error($RECOVER_E_CCASE_UP_CHECK);
	$ret_status = 1;
	last CLEANUP_STOP_CLEARCASE;
    }
    
    if($status) {
	print "Shutting down Clearcase...";
	if($MDEP_MACHINE eq 'NT') {
	    $status = &mdep_stop_clearcase_nt();
	}
	else {
	    $status = &mdep_stop_clearcase();
	}
	
	$status = &mdep_is_clearcase_up();
	if($status < 0) {
	    
	    &recover_error($RECOVER_E_CCASE_UP_CHECK);
	    $ret_status = 1;
	    last CLEANUP_STOP_CLEARCASE;
	}
	
	if($status) {

	    &recover_error("Clearcase failed to stop\n");
	    $ret_status = 1;
	    last CLEANUP_STOP_CLEARCASE;
	}
	
	$recover_cl_start_clearcase = 1;
	print "Clearcase shutdown complete\n";
	$ret_status = 0;
    }

  } # CLEANUP_STOP_CLEARCASE

    return $ret_status;

}

sub recover_start_clearcase {

    local($status);
    local($ret_status);

CLEANUP_START_CLEARCASE: {

    $status = &mdep_is_clearcase_up();
    if($status < 0) {
	
	&recover_error($RECOVER_E_CCASE_UP_CHECK);
	$ret_status = 1;
	last CLEANUP_START_CLEARCASE;
    }
    
    if(!$status) {
	
	print "Starting Clearcase...";
	if($MDEP_MACHINE eq 'NT') {
	    &mdep_start_clearcase_nt();
	}
	else {
	    &mdep_start_clearcase();
	}
	$recover_cl_start_clearcase = 0;
	
	$status = &mdep_is_clearcase_up();
	if($status < 0) {
	    
	    &recover_error($RECOVER_E_CCASE_UP_CHECK);
	    $ret_status = 1;
	    last CLEANUP_START_CLEARCASE;
	}
	
	if(!$status) {
	    
	    &recover_error("Clearcase failed to start\n");
	    $ret_status = 1;
	    last CLEANUP_START_CLEARCASE;
	}
	print "Clearcase start complete\n";
    }
  }  # CLEANUP_START_CLEARCASE

    return $rest_status;
}

sub recover_move_old_storage {

    local($stop_ccase) = @_;
    local($reply);
    local($fstr);
    local($ret_status);

CLEANUP_MOVE_OLD_STORAGE: {

    if(!&recover_st_chk("recover_st_old_storage_moved")) {

	if($stop_ccase) {

	    if(($recover_scenario == $RECOVER_SC_UP_NS_NH) ||
	       ($recover_scenario == $RECOVER_SC_DN_NS_NH)) {
		
		$fstr = sprintf($RECOVER_P_STOP_START_CCASE_OTHER_HOST,
				$recover_rgy_storage,
				$recover_rgy_host);
		$reply = &mdep_prompt($fstr,
				      $MDEP_QUIT_RETURN_TO_CONTINUE,
				      $MDEP_YES,
				      $RECOVER_H_STOP_START_CCASE_OTHER_HOST);
		
		if(lc($reply) eq $MDEP_QUIT) {
		    $ret_status = 1;
		    last CLEANUP_MOVE_OLD_STORAGE;
		}
	    }
	    else {

		if($recover_scenario == $RECOVER_SC_UP_OS_OH_NO_TEMP) {
		    $reply = &mdep_prompt($RECOVER_P_STOP_START_CCASE_THIS_HOST,
					  $MDEP_YES_HELP_QUIT,
					  $MDEP_YES,
					  $RECOVER_H_STOP_START_CCASE_FOR_IN_PLACE);
		}
		else {
		    $reply = &mdep_prompt($RECOVER_P_STOP_START_CCASE_THIS_HOST,
					  $MDEP_YES_HELP_QUIT,
					  $MDEP_YES,
					  $RECOVER_H_STOP_START_CCASE_THIS_HOST);
		}
		if(lc($reply) eq $MDEP_QUIT) {
		    $ret_status = 1;
		    last CLEANUP_MOVE_OLD_STORAGE;
		}

		if(&recover_stop_clearcase()) {
		    $ret_status = 1;
		    last CLEANUP_MOVE_OLD_STORAGE;
		}
	    }
	}
	    
	#
	# If we are a host different from the original the notification
	# to shutdown clearcase above also told the user to move the old 
	# storage on the appropriate host.
	#
	if(($recover_scenario == $RECOVER_SC_DOWN) ||
	   mdep_same_hosts($recover_rgy_host,$recover_host_name)) {

	    if(($recover_scenario == $RECOVER_SC_DOWN) &&
		-d $recover_arg_back_storage &&
		    !&recover_st_chk("recover_not_reg_data_restored")) {

		$storage = $recover_arg_back_storage;

	    }
	    else {
		

		$storage = $recover_rgy_storage;
	    }

	    while(-e $storage ) {

		#
		# Tell the user to move or remove the old storage.
		#
		$fstr = sprintf($RECOVER_P_MOVE_OLD_LOCAL_STORAGE,
				$storage);
		$reply = &mdep_prompt($fstr,
				      $MDEP_QUIT_RETURN_TO_CONTINUE,
				      $MDEP_NONE,
				      $RECOVER_H_MOVE_OR_REMOVE_OLD_STORAGE);
		
		if(lc($reply) eq $MDEP_QUIT) {
		    $ret_status = 1;
		    last CLEANUP_MOVE_OLD_STORAGE;
		}
		
		if(!-d $storage) {
		    
		    last;
		}
	    }
	}

	if($stop_ccase) {

	    if(&recover_start_clearcase()) {
		$ret_status = 1;
		last CLEANUP_MOVE_OLD_STORAGE;
	    }
	}
    }
    &recover_set_state("recover_st_old_storage_moved");
    $ret_status = 0;

  } # CLEANUP_MOVE_OLD_STORAGE

    return $ret_status;
}


$recover_sig_cleanup = 0;
sub sig_handler {
    local($sig) = @_;
    local($curfunc) = pop(@recover_cur_function);
    push(@recover_cur_function,$curfunc);

    $recover_signal_count++;

    $SIG{'INT'} = 'sig_handler';
    $SIG{'QUIT'} = 'sig_handler';
    $SIG{'TERM'} = 'sig_handler';
    $SIG{'HUP'} = 'sig_handler';
    if(!$recover_sig_cleanup) {
	$fstr = sprintf($RECOVER_N_SIGNAL_RECEIVED,$sig);
	print $fstr;
	$recover_sig_cleanup++;
	&recover_cleanup();
    }

    
    $fstr = sprintf($RECOVER_P_FINISH_CLEANUP,$sig);
    $reply = &mdep_prompt($fstr,
			  $MDEP_YES_OR_NO,
			  $MDEP_YES,
			  $MDEP_NO_HELP);
    if(lc($reply) ne $MDEP_YES) {
	exit(1);
    }
}

MAIN:

# CLEARCASE_ENABLE_ATE_MESSAGES environment variable, 
# might cause printing a lot of extra messages 
# which confuse this perl script. This is why we
# need to remove this variable from the 
# ENV array. 
delete $ENV{"CLEARCASE_ENABLE_ATE_MESSAGES"};

$CLEARCASE_LIGHT = "";
$vercmd = "$MDEP_CLEARTOOL -version";

if(&mdep_command_s(*vercmd,*cmdbuf)) {
    &param_error("Could not obtain ClearCase version information\n");
    &param_cleartool_message(*command,*cmdbuf);
    exit(1);
}

while($_ = shift(@cmdbuf)) {
    chop;

    if(/ClearCase LT/) {
        $CLEARCASE_LIGHT = TRUE;
    }
}

#open(DEBUG,">DEBUG");
#$oldfh = select(DEBUG);$| = 1;select($oldfh);
if(lc($reply) eq $MDEP_QUIT) {
    exit(1);
}
$SIG{'INT'} = 'sig_handler';
$SIG{'QUIT'} = 'sig_handler';
$SIG{'TERM'} = 'sig_handler';
$SIG{'HUP'} = 'sig_handler';

$oldfh = select(STDOUT);$| = 1;select($oldfh);
$oldfh = select(STDERR);$| = 1;select($oldfh);


&recover_set_cur_function("recover_main");

&recover_parse_arguments(@ARGV);

$reply = &mdep_prompt($RECOVER_P_PREAMBLE,
		      $MDEP_QUIT_RETURN_TO_CONTINUE,
		      $MDEP_NONE,
		      $MDEP_NO_HELP);
if(lc($reply) eq $MDEP_QUIT) {
    &recover_cleanup();
}

if(! &mdep_privileged()) {

    &recover_error("You Must be root\n");
    exit(1);
}


#
# Set the hostname before we do anything so things like lsvob do
# the right thing.
#
&recover_check_clearcase();
$recover_host_name = &mdep_get_hostname();




#
# Get the vob information
#
$recover_rgy_replica_uuid = &recover_st_chk("recover_rgy_replica_uuid");
$recover_rgy_family_uuid = &recover_st_chk("recover_rgy_family_uuid");
$recover_rgy_storage = &recover_st_chk("recover_rgy_storage");
$recover_rgy_host = &recover_st_chk("recover_rgy_host");
$recover_orig_host = &recover_st_chk("recover_orig_host");

if(($recover_rgy_replica_uuid) &&
   ($recover_rgy_family_uuid) &&
   ($recover_rgy_storage) &&
   ($recover_rgy_host)) {
    #
    # Make sure the host is right.
    # 
    if(!mdep_same_hosts($recover_orig_host,$recover_host_name)) {

	&recover_error("This restore was begun on a different host \"$recover_orig_host\"\n");
	&recover_cleanup();
    }
    #
    # This is a restart. Make sure someone hasn't created a new vob storage
    # directory with the same tag as the old tag or reused the tag to point
    # to a different storage area.
    #

    #
    # First check the tag...
    #
    $status = &recover_check_vob_info("tag",$recover_arg_vob_tag);
    if($status ne $RTN_OK) {
	
	if($status eq $RTN__VOB_INFO_INCONSISTENT) {

	    &recover_cleanup();
	}
    }
    #
    # ...then the storage path
    #
    $status = &recover_check_vob_info("storage",$recover_rgy_storage);
    if($status ne $RTN_OK) {

	if($status eq $RTN_VOB_INFO_INCONSISTENT) {
	    
	    &recover_cleanup();
	}
    }

    #
    # We have registry information.
    #
    $recover_registered = 1;
    if($recover_arg_snap_storage) {


	#
	# Make sure its the same as last time.
	#
	$prev_snap = &recover_st_chk("recover_arg_snap_storage");
	if(!mdep_path_equal($recover_arg_snap_storage,$prev_snap)) {

	    if(!$prev_snap) { $prev_snap = "NULL";}

	    $fstr = sprintf(
"The previous invocation\'s snap path is not the same as the one given.
    Previous snap path: $prev_snap,
    Give snap path: $recover_arg_snap_storage\n");
	    &recover_error($fstr);
	    &recover_cleanup();
	}

	#
	# Remove any punctuation. Makes it a bit more portable. 
	#
	($uuid_digits = $recover_rgy_replica_uuid) =~ s/\W//g;
	$recover_arg_snap_storage = 
	    $recover_arg_snap_storage.$MDEP_SEP.$uuid_digits;

	$recover_snapath_complete = 1;
    }
}
else {

    if(!&recover_get_vob_info()) {

	$recover_registered = 1;
    }
}

    
$recover_scenario = &recover_st_chk("recover_scenario");

if(!$recover_scenario) {

    $recover_scenario = &recover_get_scenario();

    if($recover_scenario <= 0) {

	if($recover_scenario = 0) {
	    $recover_do_save_state;
	}
	&recover_cleanup();
    }
    &recover_set_state("recover_scenario=$recover_scenario");
}

$status = &recover_confirm_scenario();
if($status ne $RTN_OK) {

    if($status ne $RTN_QUIT) {
	&recover_error("Unknown scenario\n");
    }
    &recover_cleanup();
}
$recover_do_save_state = 1;
#
# Determine where the recovered storage is stored. 
#
if($recover_arg_temp_storage) {

    #
    # Its been restored to a temporary staging area for processing.
    #
    $restore_storage = $recover_arg_temp_storage;

}
else {

    # 
    # Its in its final location either new or old
    #
    $restore_storage = $recover_arg_dest_storage;

}

&recover_set_state("recover_orig_host=$recover_host_name");
if($recover_rgy_replica_uuid) {
    &recover_set_state("recover_rgy_replica_uuid=$recover_rgy_replica_uuid");
}
if($recover_rgy_family_uuid) {
    &recover_set_state("recover_rgy_family_uuid=$recover_rgy_family_uuid");
}
if($recover_rgy_storage) {
    &recover_set_state("recover_rgy_storage=$recover_rgy_storage");
}
if($recover_rgy_host) {
    &recover_set_state("recover_rgy_host=$recover_rgy_host");
}
&recover_set_state("recover_arg_dest_storage=$recover_arg_dest_storage");
&recover_set_state("recover_arg_back_storage=$recover_arg_back_storage");
if($recover_arg_temp_storage) {
    &recover_set_state("recover_arg_temp_storage=$recover_arg_temp_storage");
}
if($recover_arg_snap_storage) {
    &recover_set_state("recover_arg_snap_storage=$recover_arg_snap_storage");
}
if($recover_replicated) {
    &recover_set_state("recover_replicated");
}


#
# If the vob has been left available, locked for read only access, it must
# be made unavailable at this point. 
# 
if($recover_scenario != $RECOVER_SC_DOWN) {
    
    if(($recover_scenario == $RECOVER_SC_UP_NS_OH) &&
       &recover_st_chk("recover_backup_equals_registered") &&
       !&recover_st_chk("recover_backup_overwrite_done")) {

	    #
	    # A nonsense case but one that is allowed. The target is a new 
	    # storage directory but for some reason the restored data is being 
	    # restored to the old storage location.
	    #
	    if(&recover_unregister()) {
		&recover_cleanup();
	    }
	    if(!&recover_st_chk("recover_st_old_storage_moved")) {

		$reply = &mdep_prompt($RECOVER_P_STOP_START_CCASE_THIS_HOST,
				      $MDEP_YES_HELP_QUIT,
				      $MDEP_YES,
				      $RECOVER_H_STOP_START_CCASE_THIS_HOST);
		if(lc($reply) eq $MDEP_QUIT) {
		    &recover_cleanup();
		}
                if(&recover_stop_clearcase()) {
                    &recover_cleanup();
                }
		if(&recover_start_clearcase()) {
		    &recover_cleanup();
		}
		
		if(&recover_move_old_storage(0)) {
		    &recover_cleanup();
		}
	    }
            while(! -d $recover_arg_temp_storage) {
		$fstr = sprintf($RECOVER_P_GET_RESTORED_DATA,$recover_arg_temp_storage);
		$reply = &mdep_prompt($fstr,
				      $MDEP_QUIT_RETURN_TO_CONTINUE,
				      $MDEP_NONE,
				      $MDEP_NO_HELP);
		
		if(lc($reply) eq $MDEP_QUIT) {
		    &recover_cleanup();
		}
	    }
	    &recover_set_state("recover_backup_overwrite_done");
            &recover_set_state("recover_st_old_storage_moved");
	}
   
       
    if($recover_scenario != $RECOVER_SC_UP_OS_OH_NO_TEMP) {

	$status = &recover_check_restore($recover_arg_back_storage);
	if($status ne $RTN_OK) {
	    &recover_cleanup();
	}
    }

    #
    # If a snapshot area was given merge it into the restore area. Unless
    # the restoration is in place - it will be done later.
    #
    if($recover_arg_snap_storage  &&
       ($recover_scenario != $RECOVER_SC_UP_OS_OH_NO_TEMP) &&
       (!&recover_st_chk("recover_st_snapshot_restored"))) {
	
	
	$status = &recover_restore_snapshot($recover_arg_back_storage);
	if($status ne $RTN_OK) {
	    
	    &recover_cleanup();
	}
	&recover_set_state("recover_st_snapshot_restored");
    }

    if(&recover_unregister()) {
	&recover_cleanup();
    }

    if(&recover_move_old_storage(1)) {
	&recover_cleanup();
    }

    &recover_set_state("recover_st_old_storage_moved");
} 
else { 

    if(&recover_unregister()) {
	&recover_cleanup();
    }
    
    if(!&recover_st_chk("recover_not_reg_data_restored") &&
       !&recover_st_chk("recover_st_stopped_restarted")) {

	$reply = &mdep_prompt($RECOVER_P_WAS_VOB_ON_THIS_HOST,
			      $MDEP_YES_OR_NO,
			      $MDEP_YES,
			      $RECOVER_H_STOP_START_CCASE_UNREG);


	if(lc($reply) eq $MDEP_YES) {

	    if(&recover_move_old_storage(1)) {
		&recover_cleanup();
	    }

	}
	else {

	    $reply = &mdep_prompt($RECOVER_P_STOP_START_CCASE_OTHER_HOST_UNREG,
				  $MDEP_QUIT_RETURN_TO_CONTINUE,
				  $MDEP_NONE,
				  $RECOVER_H_STOP_START_CCASE_OTHER_HOST);

	    if(lc($reply) eq $MDEP_QUIT) {
		&recover_cleanup();
	    }

	    if(&recover_move_old_storage(0)) {
		&recover_cleanup(); 
	    }
	    
	}
	
	&recover_set_state("recover_st_stopped_restarted");
    }

}


if(!&recover_st_chk("recover_already_in_place")) {

    $status = &recover_restore_storage();
    if($status ne $RTN_OK) {
	
	&recover_cleanup();
    }
    &recover_set_state("recover_already_in_place");

}


#
# Just enable it.
#
if(!&recover_st_chk("recover_replica_enabled")) {

    $status = &recover_enable_replica();
    if($status ne $RTN_OK) {
	
	&recover_cleanup();
    }
    &recover_set_state("recover_replica_enabled");
}
    
if(!&recover_st_chk("recover_snap_attribute_checked")) {
    $status = &mdep_scheduled_for_snap(*recover_arg_vob_tag,*snap_to_path);
    if($status < 0) {
	$reply = &mdep_prompt($RECOVER_E_HAS_SNAP_ATTRIBUTE,
			      $MDEP_YES_OR_NO,
			      $MDEP_NO_DEFAULT,
			      $MDEP_NO_HELP);
	
	if(lc($reply) eq $MDEP_YES) {
	    &recover_cleanup();
	}
    }
    elsif($status == 1) {
	
	$fstr = sprintf($RECOVER_P_HAS_SNAP_ATTRIBUTE,$recover_arg_vob_tag);
	$reply = &mdep_prompt($fstr,
			      $MDEP_YES_OR_NO,
			      $MDEP_NO_DEFAULT,
			      $MDEP_NO_HELP);
	if(lc($reply) eq $MDEP_QUIT) {
	    &recover_cleanup();
	}
	if(lc($reply) eq $MDEP_YES) {
	    if($recover_replicated) {
		$reply = &mdep_prompt($RECOVER_P_VERIFY_REMOVE_SNAP,
				      $MDEP_YES_OR_NO,
				      $MDEP_NO,
				      $MDEP_NO_HELP);

		if(lc($reply) eq $MDEP_QUIT) {
		    &recover_cleanup();
		}
		if(lc($reply) eq $MDEP_YES) {

		    $status = &mdep_remove_snap_attribute($recover_arg_vob_tag,
						  $recover_arg_dest_storage,
						  "vob_restore");
		    if($status < 1) { 
			&recover_error($RECOVER_E_RM_SNAP_ATTRIBUTE_FAILED );
			&recover_cleanup();
		    }
		}
	    }
	    else {
		$status = &mdep_remove_snap_attribute($recover_arg_vob_tag,
					      $recover_arg_dest_storage,
					      "vob_restore");
		if($status < 1) { 
		    &recover_error($RECOVER_E_RM_SNAP_ATTRIBUTE_FAILED );
		    &recover_cleanup();
		}
	    }
	}
    }
}
&recover_set_state("recover_snap_attribute_checked");


if(!&recover_st_chk("recover_checkvob_done")) {
 
    $status = &recover_check_the_vob();
    if($status < 0) {
        &recover_cleanup();
    }
    if($status > 0)  {
        &recover_alldone();
    }
}

&recover_set_state("recover_checkvob_done");



&recover_alldone();


